#!/usr/bin/env python
# pylint: disable=too-many-locals,too-many-arguments,superfluous-parens,import-error,invalid-name,arguments-differ,
"""
Created on 2019/4/26

@author: yyang
"""
import math
import os

import struct
from time import sleep

from tool.device.nvme.ioctl import IO, ADMIN, IOCTL
from tool.device.nvme.struct import *
from tool.device.nvme.struct.hmb import HMB
from utils import log
from tool.device.nvme.buf import *
from tool.device.nvme.utility_vu import *


class NVMe(IO, ADMIN):
    def get_flbas(self, nsid=1, cntid=0):
        id_ns = self.identify_ns(nsid=nsid, cntid=cntid)
        return id_ns.FLBAS & 0xf

    def format_as(self, lbads=4096, ms=0, nsid=1, cntid=0, **kwargs):
        id_ns = self.identify_ns(nsid=nsid, cntid=cntid)
        for i, lbaf in enumerate(id_ns.LBAF):
            if lbads == 2 ** lbaf.LBADS and ms == lbaf.MS:
                self.format(nsid, i, **kwargs)
                return

        raise AssertionError('Invalid lbaf! lbads: {}, ms: {}'.format(lbads, ms))

    def attach_ns(self, cntid, nsid=1, **kwargs):
        ret = self.namespace_attachment(0, cntid, nsid=nsid, **kwargs)
        sleep(1)
        # some OS need reset to find block device
        # self.reset()
        # sleep(1)
        return ret

    def detach_ns(self, cntid, nsid=1, **kwargs):
        return self.namespace_attachment(1, cntid, nsid=nsid, **kwargs)

    def create_ns(self, nsze=0, ncap=0, flbas=0, dps=0, nmic=0, **kwargs):
        return self.namespace_management(0, nsze=nsze, ncap=ncap, flbas=flbas, dps=dps, nmic=nmic, **kwargs)

    def delete_ns(self, nsid, **kwargs):
        return self.namespace_management(1, nsid=nsid, **kwargs)

    def get_log_error_info(self, cntid=0):
        id_ctrl = self.identify_ctrl(cntid=cntid)
        dptr = (ErrorInformation * (id_ctrl.ELPE + 1))()
        self.get_log(dptr, lid=0x1)
        return dptr

    def get_log_smart_health_info(self):
        dptr = SmartHealthInformation()
        self.get_log(dptr, lid=0x2)
        return dptr

    def get_log_fw_slot_info(self):
        dptr = FirmwareSlotInformation()
        self.get_log(dptr, lid=0x3)
        return dptr

    def get_log_change_ns_list(self):
        dptr = DeviceSelfTest()
        self.get_log(dptr, lid=0x4)
        return dptr

    def get_log_cmd_supported_effects(self):
        dptr = CommandsSupportAndEffects()
        self.get_log(dptr, lid=0x5)
        return dptr

    def self_test(self, stc, testaction=0):
        pass

    def get_log_device_self_test(self):
        dptr = SelfTestLog()
        self.get_log(dptr, lid=0x6)
        return dptr

    def get_log_telemetry_host_init_log(self, lsp=0x0):
        dptr = TelemetryHostInitiated()
        self.get_log(dptr, lid=0x7, lsp=lsp)
        return dptr

    def get_log_telemetry_ctrl_init_log(self):
        dptr = TelemetryControllerInitiated()
        self.get_log(dptr, lid=0x8)
        return dptr

    def get_log_reservation_notification(self):
        dptr = ReservationNotificationLog()
        self.get_log(dptr, lid=0x80)
        return dptr

    def get_log_sanitize_status(self):
        dptr = SanitizeStatusLog()
        self.get_log(dptr, lid=0x81)
        return dptr

    def identify_ns(self, nsid=1, cntid=0):
        return self.identify(nsid=nsid, cns=0x0, cntid=cntid)[1]

    def identify_ctrl(self, cntid=0):
        return self.identify(nsid=0, cns=0x1, cntid=cntid)[1]

    def identify_active_ns_list(self, nsid=1, cntid=0):
        return self.identify(nsid=nsid, cns=0x2, cntid=cntid)[1]

    def identify_allocated_ns_list(self, nsid=1, cntid=0):
        return self.identify(nsid=nsid, cns=0x10, cntid=cntid)[1]

    def identify_attached_ctrl_list(self, nsid=1, cntid=0):
        return self.identify(nsid=nsid, cns=0x12, cntid=cntid)[1]

    def identify_existed_ctrl_list(self, cntid=0):
        return self.identify(nsid=0, cns=0x13, cntid=cntid)[1]

    def identify_primary_ctrl_cap(self, cntid=0):
        return self.identify(nsid=0, cns=0x14, cntid=cntid)[1]

    def identify_secondary_ctrl_list(self, cntid=0):
        return self.identify(nsid=0, cns=0x15, cntid=cntid)[1]

    def get_features_arbitration(self, sel=0):
        return self.get_features(FeatureID.Arbitration, sel)[1]

    def set_features_arbitration(self, save=0, ab=0, lpw=0, mpw=0, hpw=0, **kwargs):
        cdw11 = Arbitration()
        cdw11.AB = ab
        cdw11.LPW = lpw
        cdw11.MPW = mpw
        cdw11.HPW = hpw
        return self.set_features(0x1, save, cdw11.DWORD, **kwargs)

    def get_features_power_management(self, sel=0, **kwargs):
        return self.get_features(FeatureID.PowerManagement, sel, **kwargs)[1]

    def set_features_power_management(self, save=0, ps=0, wh=0, **kwargs):
        cdw11 = PowerManagement()
        cdw11.PS = ps
        cdw11.WH = wh
        return self.set_features(0x2, save, cdw11.DWORD, **kwargs)

    def get_features_lba_range_type(self, sel=0, nsid=1, **kwargs):
        return self.get_features(FeatureID.LBARangeType, sel, nsid=nsid, **kwargs)[1]

    def set_features_lba_range_type(self, lrt, save=0, nsid=1, **kwargs):
        cdw11 = LBARangeType()
        cdw11.NUM = len(lrt) - 1

        dptr = (LBARangeTypeEntry * 64)()
        for i, _lrt in enumerate(lrt):
            if isinstance(_lrt, dict):
                for key, val in _lrt.items():
                    if key == 'GUID':
                        setattr(dptr[i].GUID, 'LOW', val & 0xffffffffffffffff)
                        setattr(dptr[i].GUID, 'HIGH', val >> 64)
                    else:
                        setattr(dptr[i], key, val)
            else:
                for key, _ in getattr(_lrt, '_fields_'):
                    setattr(dptr[i], key, getattr(_lrt, key))
        return self.set_features(FeatureID.LBARangeType, save, cdw11.DWORD, dptr=dptr, nsid=nsid, **kwargs)

    def get_features_temperature_threshold(self, sel=0, **kwargs):
        return self.get_features(FeatureID.TemperatureThreshold, sel, **kwargs)[1]

    def set_features_temperature_threshold(self, save=0, tmpth=0, tmpsel=0, thsel=0, **kwargs):
        cdw11 = TemperatureThreshold()
        cdw11.TMPTH = tmpth
        cdw11.TMPSEL = tmpsel
        cdw11.THSEL = thsel
        return self.set_features(FeatureID.TemperatureThreshold, save, cdw11.DWORD, **kwargs)

    def get_features_host_memory_buffer(self, sel=0, **kwargs):
        return self.get_features(FeatureID.HostMemoryBuffer, sel, **kwargs)[2]

    def set_features_host_memory_buffer(self, save=0, ehm=0, mr=0, esize=0, hdmlec=0, **kwargs):
        cdw11 = HostMemoryBuffer()
        cdw11.EHM = ehm
        cdw11.MR = mr
        # get CC.MPS and init HMB size
        ccmps = 4096
        hsize = 0
        # config Descriptor List
        dptr = HostMemoryBufferDescriptorList()
        for i in range(hdmlec):
            # set entry
            dptr.ENTRY[i].BSIZE = esize
            dptr.ENTRY[i].BADD = addressof
            # cal HMB size
            hsize += (esize * ccmps)
        # get dwords info
        hmdlla = addressof(dptr) & 0xffffffff
        hmdlua = (addressof(dptr) >> 32) & 0xffffffff

        return self.set_features(FeatureID.HostMemoryBuffer, save, cdw11.DWORD, cdw12=hsize, cdw13=hmdlla,
                                 cdw14=hmdlua, cdw15=hdmlec, **kwargs)

    def get_features_non_operational_power_state_config(self, sel=0, **kwargs):
        return self.get_features(FeatureID.NonOperationalPowerStateConfig, sel, **kwargs)[2]

    def set_features_non_operational_power_state_config(self, nopsc, save=0, **kwargs):
        dptr = NonOperationalPowerStateConfig()
        dptr.NonOperationalPowerStateConfig = nopsc
        return self.set_features(FeatureID.NonOperationalPowerStateConfig, save, 0, dptr=dptr, **kwargs)

    def get_features_error_recovery(self, sel=0, nsid=1, **kwargs):
        return self.get_features(FeatureID.ErrorRecovery, sel, nsid=nsid, **kwargs)[1]

    def set_features_error_recovery(self, save=0, tler=0, dulbe=0, nsid=1, **kwargs):
        cdw11 = ErrorRecovery()
        cdw11.TLER = tler
        cdw11.DULBE = dulbe
        return self.set_features(FeatureID.ErrorRecovery, save, cdw11.DWORD, nsid=nsid, **kwargs)

    def get_features_volatile_write_cache(self, sel=0, **kwargs):
        return self.get_features(FeatureID.VolatileWriteCache, sel, **kwargs)[1]

    def set_features_volatile_write_cache(self, save=0, wce=0, **kwargs):
        cdw11 = VolatileWriteCache()
        cdw11.WCE = wce
        return self.set_features(FeatureID.VolatileWriteCache, save, cdw11.DWORD, **kwargs)

    def get_features_number_of_queues(self, sel=0, **kwargs):
        return self.get_features(FeatureID.NumberOfQueues, sel, **kwargs)[1]

    def set_features_number_of_queues(self, lrt, save=0, **kwargs):
        cdw11 = NumberOfQueues()
        cdw11.NSQR = len(lrt) - 1
        cdw11.NCQR = len(lrt) - 1

        dptr = (LBARangeTypeEntry * 64)()
        for i, _lrt in enumerate(lrt):
            if isinstance(_lrt, dict):
                for key, val in _lrt.items():
                    setattr(dptr[i], key, val)
            else:
                for key, _ in getattr(_lrt, '_fields_'):
                    setattr(dptr[i], key, getattr(_lrt, key))

        return self.set_features(FeatureID.NumberOfQueues, save, cdw11.DWORD, **kwargs)

    def get_features_interrupt_coalescing(self, sel=0, **kwargs):
        return self.get_features(FeatureID.InterruptCoalescing, sel, **kwargs)[1]

    def set_features_interrupt_coalescing(self, save=0, thr=0, time=0, **kwargs):
        cdw11 = InterruptCoalescing()
        cdw11.THR = thr
        cdw11.TIME = time
        return self.set_features(FeatureID.InterruptCoalescing, save, cdw11.DWORD, **kwargs)

    def get_features_interrupt_vector_configuration(self, sel=0, iv=1, **kwargs):
        cdw11 = InterruptVectorConfiguration()
        cdw11.IV = iv
        return self.get_features(FeatureID.InterruptVectorConfiguration, sel, cdw11=cdw11.DWORD, **kwargs)[1]

    def set_features_interrupt_vector_configuration(self, save=0, iv=0, cd=0, **kwargs):
        cdw11 = InterruptVectorConfiguration()
        cdw11.IV = iv
        cdw11.CD = cd
        return self.set_features(FeatureID.InterruptVectorConfiguration, save, cdw11.DWORD, **kwargs)

    def get_features_write_atomicity_normal(self, sel=0, **kwargs):
        return self.get_features(FeatureID.WriteAtomicityNormal, sel, **kwargs)[1]

    def set_features_write_atomicity_normal(self, save=0, dn=0, **kwargs):
        cdw11 = WriteAtomicityNormal()
        cdw11.DN = dn
        return self.set_features(FeatureID.WriteAtomicityNormal, save, cdw11.DWORD, **kwargs)

    def get_features_asynchronous_event_configuration(self, sel=0, **kwargs):
        return self.get_features(FeatureID.AsynchronousEventConfiguration, sel, **kwargs)[1]

    def set_features_asynchronous_event_configuration(self, save=0, shcw=0, nan=0, fan=0, tln=0, **kwargs):
        cdw11 = AsynchronousEventConfiguration()
        cdw11.SHCW = shcw
        cdw11.NAN = nan
        cdw11.FAN = fan
        cdw11.TLN = tln
        return self.set_features(FeatureID.AsynchronousEventConfiguration, save, cdw11.DWORD, **kwargs)

    def get_features_autonomous_power_state_transition(self, sel=0, **kwargs):
        return self.get_features(FeatureID.AutonomousPowerStateTransition, sel, **kwargs)[1]

    def set_features_autonomous_power_state_transition(self, save=0, apste=0, **kwargs):
        cdw11 = AutonomousPowerStateTransition()
        cdw11.APSTE = apste
        return self.set_features(FeatureID.AutonomousPowerStateTransition, save, cdw11.DWORD, **kwargs)

    def get_features_software_progress_marker(self, sel=0, **kwargs):
        return self.get_features(FeatureID.SoftwareProgressMarker, sel, **kwargs)[2]

    def set_features_software_progress_marker(self, save=0, pbslc=0, **kwargs):
        cdw11 = SoftwareProgressMarker()
        cdw11.PBSLC = pbslc
        return self.set_features(FeatureID.SoftwareProgressMarker, save, cdw11.DWORD, **kwargs)

    def get_features_host_identifier(self, sel=0, **kwargs):
        return self.get_features(FeatureID.HostIdentifier, sel, **kwargs)[2]

    def set_features_host_identifier(self, exhid, hostidl, hostidh, save=0, **kwargs):
        cdw11 = HostIdentifier()
        cdw11.EXHID = exhid
        dptr = HostIdentifierEntry()
        dptr.HOSTIDL = hostidl
        dptr.HOSTIDH = hostidh
        return self.set_features(FeatureID.HostIdentifier, save, cdw11.DWORD, dptr=dptr, **kwargs)

    def get_features_reservation_notification_mask(self, sel=0, **kwargs):
        return self.get_features(FeatureID.ReservationNotificationMask, sel, nsid=1, **kwargs)[2]

    def set_features_reservation_notification_mask(self, save=0, regpre=0, resrel=0, respre=0, **kwargs):
        cdw11 = ReservationNotificationMask()
        cdw11.REGPRE = regpre
        cdw11.RESREL = resrel
        cdw11.RESPRE = respre
        return self.set_features(FeatureID.ReservationNotificationMask, save, **kwargs)

    def get_features_reservation_persistence(self, sel=0, **kwargs):
        return self.get_features(FeatureID.ReservationPersistence, sel, nsid=1, **kwargs)[2]

    def set_features_reservation_persistence(self, save=0, ptpl=0, **kwargs):
        cdw11 = ReservationPersistence()
        cdw11.PTPL = ptpl
        return self.set_features(FeatureID.ReservationPersistence, save, nsid=1, **kwargs)

    def get_features_host_controlled_thermal_management(self, sel=0, **kwargs):
        return self.get_features(FeatureID.HostControlledThermalManagement, sel, **kwargs)[2]

    def set_features_host_controlled_thermal_management(self, save=0, tmt2=0, tmt1=0, **kwargs):
        cdw11 = HostControlledThermalManagement()
        cdw11.TMT2 = tmt2
        cdw11.TMT1 = tmt1
        return self.set_features(FeatureID.HostControlledThermalManagement, save, **kwargs)

    def get_features_timestamp(self, sel=0, **kwargs):
        return self.get_features(FeatureID.Timestamp, sel, **kwargs)[2]

    def set_features_timestamp(self, timestamp, save=0, **kwargs):
        dptr = Timestamp()
        dptr.TIMESTAMP = timestamp
        return self.set_features(FeatureID.Timestamp, save, 0, dptr=dptr, **kwargs)

    def get_features_sanitize_config(self, sel=0, **kwargs):
        return self.get_features(FeatureID.SanitizeConfig, sel, **kwargs)[2]

    def set_features_sanitize_config(self, save=0, nodrm=0, **kwargs):
        cdw11 = SanitizeConfig()
        cdw11.NODRM = nodrm
        return self.set_features(FeatureID.SanitizeConfig, save, **kwargs)

    def get_features_nonoperational_power_state_config(self, sel=0, **kwargs):
        return self.get_features(FeatureID.NonOperationalPowerStateConfig, sel, **kwargs)[2]

    def set_features_nonoperational_power_state_config(self, save=0, noppme=0, **kwargs):
        cdw11 = NonOperationalPowerStateConfig()
        cdw11.NOPPME = noppme
        return self.set_features(FeatureID.NonOperationalPowerStateConfig, save, **kwargs)


class CNEX(NVMe):
    def fw_inject_uecc_start(self):
        cmd = NVMePassthruCmd()
        cmd.OPCODE = 0xC0
        cmd.CDW[10] = 0x1
        cmd.CDW[15] = 0x14
        return self.admin_passthru(cmd)

    def fw_inject_uecc_stop(self):
        cmd = NVMePassthruCmd()
        cmd.OPCODE = 0xC0
        cmd.CDW[10] = 0x0
        cmd.CDW[15] = 0x14
        return self.admin_passthru(cmd)

    def read_register(self, addr):
        dptr = (c_uint32 * 1024)()
        cmd = NVMePassthruCmd()
        cmd.OPCODE = 0xC0
        cmd.CDW[10] = 0x2
        cmd.CDW[11] = addr
        cmd.CDW[15] = 0x14
        cmd.DATA = addressof(dptr)
        cmd.DATA_LEN = sizeof(dptr)
        self.admin_passthru(cmd)
        return dptr[0]

    def write_register(self, addr, value):
        cmd = NVMePassthruCmd()
        cmd.OPCODE = 0xC0
        cmd.CDW[10] = 0x3
        cmd.CDW[11] = addr
        cmd.CDW[12] = value
        cmd.CDW[15] = 0x14
        return self.admin_passthru(cmd)

    def rdma_read_list(self, offset, ndw=1, nsid=1):
        length = math.ceil(ndw / 1024) * 1024
        data = (c_uint32 * length)()

        cmd = NVMePassthruCmd()
        cmd.OPCODE = OPCODE.CNEX_READ_MEM
        cmd.NSID = nsid
        cmd.CDW[10] = offset & 0xffffffff
        cmd.CDW[11] = offset >> 32
        cmd.CDW[12] = ndw - 1
        cmd.DATA = addressof(data)
        cmd.DATA_LEN = length * 4

        return self.io_passthru(cmd), data[0:ndw]

    def rdma_write_list(self, offset, array, nsid=1):
        ndw = len(array)
        length = math.ceil(ndw / 1024) * 1024
        data = (c_uint32 * length)(*array)

        cmd = NVMePassthruCmd()
        cmd.OPCODE = OPCODE.CNEX_WRITE_MEM
        cmd.NSID = nsid
        cmd.CDW[10] = offset & 0xffffffff
        cmd.CDW[11] = offset >> 32
        cmd.CDW[12] = ndw - 1
        cmd.DATA = addressof(data)
        cmd.DATA_LEN = length * 4

        return self.io_passthru(cmd)

    def rdma_read(self, offset, nsid=1):
        _, value = self.rdma_read_list(offset, ndw=1, nsid=nsid)
        return value[0]

    def rdma_write(self, offset, value, nsid=1):
        return self.rdma_write_list(offset, [value], nsid=nsid)

    def get_bad_block(self, dptr, lpol=0, lpou=0):
        return self.get_log(dptr, lid=0xcb, lpol=lpol, lpou=lpou)

    def clear_gbb(self, gbb, nsid=0xffffffff):
        cmd = NVMePassthruCmd()
        cmd.OPCODE = OPCODE.CNEX_CLEAR_GBB
        cmd.NSID = nsid
        cmd.CDW[10] = gbb
        cmd.CDW[12] = 0x84

        return self.admin_passthru(cmd)

    def aging_era_nand(self):
        cmd = NVMePassthruCmd()
        cmd.OPCODE = OPCODE.AGING_MF
        cmd.CDW[12] = 0x4
        return self.admin_passthru(cmd)

    def aging_scan_fbb(self):
        cmd = NVMePassthruCmd()
        cmd.OPCODE = OPCODE.AGING_MF
        cmd.CDW[12] = 0x3
        return self.admin_passthru(cmd)

    def aging_era_dirty(self):
        cmd = NVMePassthruCmd()
        cmd.OPCODE = OPCODE.AGING_MF
        cmd.CDW[12] = 0xb
        return self.admin_passthru(cmd)

    def read_retry_ram(self, mode=0):
        tmp = mode % 2
        cmd = NVMePassthruCmd()
        cmd.OPCODE = OPCODE.CNEX_VU_COMMAND
        cmd.CDW[15] = 0x2 + tmp
        return self.admin_passthru(cmd)


class Tahoe(NVMe):
    OFFSET_REGISTER = 0x100000

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.dll = CDLL(os.path.realpath(os.path.join(os.path.dirname(__file__), "..", "dll", 'buf.so')))
        self.dll.nvme_alloc.restype = c_uint64
        vu_class = VUProjectType.from_name(self.__class__.__name__+"VU")
        self.vu_cmd = vu_class(self)

    def read_register(self, addr):
        return self.vu_cmd.get_memory(addr + self.OFFSET_REGISTER)[0]

    def write_register(self, addr, value):
        dptr = (c_uint32 * 128)(value)
        self.vu_cmd.set_memory(addr + self.OFFSET_REGISTER, dptr, length=1)

    def get_average_power_consumption(self, duration=60, delay=1):
        pc = 0
        max_cre = 0
        min_cre = 0
        avg_cre = 0
        for _ in range(duration):
            data = self.vu_cmd.get_power_consumption()
            pc += data[0]
            max_cre += data[1]
            min_cre += data[2]
            avg_cre += data[3]
            sleep(delay)
        return pc/duration, max_cre/duration, min_cre/duration, avg_cre/duration

    def check_security(self, offset, length):
        tmp = self.vu_cmd.get_drive_security_state()
        if length == 1:
            return tmp[offset]
        else:
            sum = 0
            a = tmp[offset:(offset+length)]
            for i in range(length):
                sum = sum + (a[i] << (i * 8))
            return sum

    def get_vendor_uniquelog(self, length, offset):
        dptr = (c_uint8 * 512)()
        self.get_log(dptr, lid=0xDE)
        a = dptr[offset:(offset + length)]
        sum = 0
        for i in range(length):
            sum = sum + (a[i] << (i * 8))
        return sum

    def check_int_type(self):
        return self.vu_cmd.get_int_type()

    def check_data(self, val):
        if val:
            return val
        return ""

    def get_info(self):
        id_ctrl_buffer = Ctype(self.identify_ctrl())
        dev_info = {}
        ssd_config_dic = {}
        drive_life_info_dic = {}
        test_fw_config_dic = {}
        fw_repo = self.check_data(re.findall('http\S+git', id_ctrl_buffer.get('VS'))[0])
        fw_branch_name = self.check_data(re.findall(".*git(.*)commit.*", id_ctrl_buffer.get('VS'))[0])
        fw_private_revision = self.check_data(re.findall('commit (\w+)', id_ctrl_buffer.get('VS'))[0])
        tnvmecap = self.check_data(id_ctrl_buffer.get('TNVMCAP'))
        fw_public_revision = self.check_data(id_ctrl_buffer.get('FR'))
        vid = self.check_data(id_ctrl_buffer.get('VID'))
        drive_sn = self.check_data(id_ctrl_buffer.get('SN'))
        drive_pn = self.check_data(id_ctrl_buffer.get('MN'))
        security_type = "ISE" if self.check_security(2, 1) else "SED"
        security_status = "security enable" if self.check_security(3, 1) else "security disable"
        int_type = self.check_data(self.check_int_type())
        life_cycle_state = self.check_security(16, 4)
        count_grown_defects = self.check_data(self.get_vendor_uniquelog(4, 144))
        count_primary_defects = self.check_data(self.get_vendor_uniquelog(4, 152))
        nand_max_erase_count = self.check_data(self.get_vendor_uniquelog(2, 112))
        nand_min_erase_count = self.check_data(self.get_vendor_uniquelog(2, 96))
        nand_avg_erase_count = self.check_data(self.get_vendor_uniquelog(2, 104))
        ssd_config_dic['drive_sn'] = drive_sn
        ssd_config_dic['drive_pn'] = drive_pn
        ssd_config_dic['vid'] = vid
        ssd_config_dic['drive_tnvmcap'] = tnvmecap
        ssd_config_dic['security_status'] = security_status
        ssd_config_dic['security_type'] = security_type
        ssd_config_dic['int_type'] = int_type
        drive_life_info_dic['life_cycle_state'] = life_cycle_state
        drive_life_info_dic['count_grown_defects'] = count_grown_defects
        drive_life_info_dic['count_primary_defects'] = count_primary_defects
        drive_life_info_dic['nand_max_erase_count'] = nand_max_erase_count
        drive_life_info_dic['nand_min_erase_count'] = nand_min_erase_count
        drive_life_info_dic['nand_avg_erase_count'] = nand_avg_erase_count
        test_fw_config_dic['fw_public_revision'] = fw_public_revision
        test_fw_config_dic['fw_repo'] = fw_repo
        test_fw_config_dic['fw_private_revision'] = fw_private_revision
        test_fw_config_dic['fw_branch_name'] = fw_branch_name
        dev_info['ssd_config_dic'] = ssd_config_dic
        dev_info['drive_life_info_dic'] = drive_life_info_dic
        dev_info['test_fw_config_dic'] = test_fw_config_dic
        return dev_info



import os
from openpyxl import load_workbook


class ExcelHandle(object):

    def __init__(self):
        pass

    @staticmethod
    def get_ogt_config(excel_file):
        ogt_config = list()
        wb = load_workbook(excel_file)
        sheet = wb.active
        for index in range(2, sheet.max_row+1):
            ip = sheet.cell(index, 1).value
            okg_config = sheet.cell(index, 2).value
            if ip not in ["", None] and okg_config not in ["", None]:
                item = {
                    "ip": ip,
                    "ogt_config": okg_config,
                }
                ogt_config.append(item)
        return ogt_config



#!/usr/bin/env python

# !/usr/bin/env python
"""
This is SSH commands

@author: Yingjun Yang
"""
import os
import sys
import binascii
import socket
import time

import paramiko

from utils import log


class SSH(object):
    def __init__(self, hostname, port=22, username='root', password='nvme', mode='rdma'):
        self._hostname = hostname
        self._port = port
        self._mac = None
        self._username = username
        self._password = password
        self._mode = mode

        self._ssh = paramiko.SSHClient()
        self._ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        self._platform = None
        self._uptime = None
        self._lib_path = '/tmp'
        self._lib_name = 'lib.py'
        self._device = '/dev/nvme'

    @property
    def _local_time(self):
        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))

    @property
    def platform(self):
        return self._platform

    @property
    def uptime(self):
        return self._uptime

    @property
    def mac(self):
        return self._mac

    def _get_mac(self):
        cmd = 'python -c "' \
              'import re, uuid; ' \
              'print(\'-\'.join(re.findall(r\'.{2}\', uuid.uuid1().hex[-12:].upper())))"'
        status, output = self.command(cmd, cmdline=False, console=False)
        if status:
            log.ERR(output)
            raise RuntimeError('Get MAC address failed!!')

        return output.strip()

    def open(self, timeout=30, interval=3):
        """
        Open ssh connection
        :param timeout:  timeout (in seconds) for open
        :param interval: timeout (in seconds) for ssh connect
        :return:
        """
        log.INFO("Connecting %s:%d", self._hostname, self._port)

        tm_bgn = time.time()
        while True:
            try:
                self._ssh.connect(self._hostname,
                                  port=self._port,
                                  username=self._username,
                                  password=self._password,
                                  timeout=interval)
                break
            except KeyboardInterrupt:
                raise
            except Exception:
                pass

            if time.time() - tm_bgn >= timeout:
                raise RuntimeError("Connect {}:{} timeout({}s)".format(self._hostname, self._port, timeout))

        self._platform = self._get_platform()
        self._uptime = self._get_uptime()
        self._mac = self._get_mac()

    def close(self):
        self._ssh.close()

    def __del__(self):
        self.close()

    def _get_platform(self):
        """
        Get platform of remote PC
        :return:
        """
        status, output = self.command(
            'python -c "import platform; print(platform.platform())"', cmdline=False, console=False)
        if status:
            log.ERR(output)
            raise RuntimeError("Get platform of remote system failed!")

        platform = output.strip()
        log.INFO(platform)
        return platform

    def _get_uptime(self):
        """
        Get boot time of remote PC
        :return:
        """
        status, output = self.command(
            'python -c "import psutil; print(psutil.boot_time())"', cmdline=False, console=False)
        if status:
            log.ERR(output)
            raise RuntimeError('Get uptime of remote system failed!')

        # log.INFO(datetime.datetime.fromtimestamp(uptime).strftime("%Y-%m-%d %H:%M:%S"))
        log.INFO('Up time: %.1fs', time.time() - float(output))
        return float(output)

    def is_active(self):
        """
        Check the connection of SSH is active or not
        :return:
        """
        if self._ssh:
            transport = self._ssh.get_transport()
            if transport:
                return transport.is_alive()
        return False

    def sftp_put(self, filename, filepath):
        """
        SFTP put file to remote path
        :param filename:
        :param filepath:
        :return:
        """
        sftp = self._ssh.open_sftp()
        sftp.put(filename, filepath)
        sftp.close()

    def is_exist(self, path):
        sftp = self._ssh.open_sftp()
        try:
            sftp.stat(path)
            sftp.close()
            return True
        except IOError:
            sftp.close()
            return False

    def make_dir(self, path):
        sftp = self._ssh.open_sftp()
        sftp.mkdir(path)
        sftp.close()

    def remove_dir(self, path):
        sftp = self._ssh.open_sftp()
        sftp.rmdir(path)
        sftp.close()

    def __get_all_files_in_local_dir(self, local_dir):
        all_files = list()
        files = os.listdir(local_dir)
        for file_name in files:
            file_ = os.path.join(local_dir, file_name)
            if os.path.isdir(file_):
                if file_ not in [os.path.join(local_dir, ".git"), os.path.join(local_dir, ".idea")]:
                    all_files.append((file_, "folder"))
                    all_files.extend(self.__get_all_files_in_local_dir(file_))
            else:
                all_files.append((file_, "file"))
        return all_files

    def sftp_put_dir(self, local_dir, remote_dir):
        sftp = self._ssh.open_sftp()
        all_files = self.__get_all_files_in_local_dir(local_dir)
        for item in all_files:
            rel_path = os.path.relpath(item[0], local_dir)
            remote_path = os.path.join(remote_dir, rel_path)
            remote_path = remote_path.replace("\\", "/")
            if item[1] == "file":
                sftp.put(item[0], remote_path)
            else:
                sftp.mkdir(remote_path)
        sftp.close()

    def execute(self, cmd, cmdline=True, timeout=None):
        """
        Execute command without waiting result
        :param cmd:     the command
        :param cmdline: print the command
        :param timeout: an optional timeout (in seconds) for the command
        :return:
        """
        if cmdline:
            log.INFO(cmd)

        if not self.is_active():
            self.open()

        self._ssh.exec_command(cmd, timeout=timeout)

    def command(self, cmd, cmdline=True, timeout=None, console=True):
        """
        Run command through ssh
        :param cmd:     the command
        :param cmdline: print the command
        :param timeout: an optional timeout (in seconds) for the command
        :param console: output to console
        :return:
        """
        if cmdline:
            log.INFO(cmd)

        if not self.is_active():
            self.open()

        _, stdout, stderr = self._ssh.exec_command(cmd, timeout=timeout)
        output = str()
        while True:
            line = stdout.readline()
            if not line:
                break

            output += line
            if console:
                sys.stdout.write(line)

        status = stdout.channel.recv_exit_status()
        output += stderr.read().decode('utf-8')
        return status, output

    def command_without_result(self, cmd, cmdline=True, timeout=None):
        if cmdline:
            log.INFO(cmd)
        if not self.is_active():
            self.open()
        self._ssh.exec_command(cmd, timeout=timeout)

    def list_disk(self):
        """
        List disk of remote PC
        :return:
        """
        status, output = self.command(
            'python -c "import psutil; print(psutil.disk_io_counters(perdisk=True))"',
            cmdline=False, console=False)
        if status:
            log.ERR(output)
            raise RuntimeError('List disk failed, status: {:#x}'.format(status))
        return output

    def reboot(self, overtime=30, option=''):
        """
        Reboot controlled computer
        :param overtime: overtime (in seconds) for reboot
        :param option:   option for reboot
        :return:
        """
        cmd = "reboot" if 'Linux' in self.platform else 'shutdown -r -t 1'
        if option:
            cmd = '{} {}'.format(cmd, option)

        self.execute(cmd, cmdline=True)
        self.close()
        time.sleep(overtime)

    def shutdown(self, overtime=30, option='', rmmod=True):
        """
        Shutdown controlled computer
        :param overtime: overtime (in seconds) for shutdown
        :param option:   option for shutdown
        :param rmmod:   rmmod nvme before power off
        :return:
        """
        if 'Linux' in self.platform:
            if rmmod:
                cmd = 'rmmod nvme'
                status, output = self.command(cmd, cmdline=True)
                assert status == 0, '{} failed!\nStatus: {:#x}\n{}'.format(cmd, status, output)

            cmd = "poweroff"
        else:
            cmd = 'shutdown -s -t 1'

        if option:
            cmd = '{} {}'.format(cmd, option)

        self.execute(cmd, cmdline=True)
        self.close()
        time.sleep(overtime)



import os
import subprocess
from test_framework.test_engine.special_parameter import Parameters
from test_framework.state import State


class OakgateEngine(object):

    def __init__(self):
        self.orig_log_folders = None
        self.latest_log_folders = None
        self.root_path = os.environ["working_path"]
        self.run_path = os.path.join(self.root_path, "run.py")
        self.logs_path = os.path.join(self.root_path, "Logs")
        self.create_logs_folder()
        self.parm = Parameters()

    def generate_command(self, parameters):
        command_line = str()
        image_parameters = self.parm.generate_redtail_images(parameters)
        parameters.update(image_parameters)
        parameters = self.parm.pop_parm(parameters, "volume")
        parameters = self.parm.pop_parm(parameters, "base_path")
        for key, value in parameters.items():
            temp_command = "--{} {} ".format(key, value)
            command_line = command_line + temp_command
        command_ = "cd {} && python run.py {}".format(self.root_path, command_line)
        print(parameters, command_)
        return command_

    def create_logs_folder(self):
        if os.path.exists(self.logs_path) is False:
            os.mkdir(self.logs_path)

    def get_new_log(self, test_case):
        self.latest_log_folders = os.listdir(self.logs_path)
        test_name = test_case.split('.')[0]
        new_logs = list()
        for item in self.latest_log_folders:
            if item not in self.orig_log_folders:
                if os.path.isdir(os.path.join(self.logs_path, item)):
                    if test_name in item:
                        new_logs.append(item)
        return new_logs

    def get_orig_logs(self):
        self.orig_log_folders = os.listdir(self.logs_path)

    def get_logs(self, log_path):
        log_content = str()
        if os.path.exists(log_path):
            log = open(log_path)
            log_content = log.read()
            log.close()
        return log_content

    def get_test_log(self, test_case):
        logs = list()
        new_logs = self.get_new_log(test_case)
        for log_folder in new_logs:
            temp_folder = os.path.join(self.logs_path, log_folder)
            for log_file in os.listdir(temp_folder):
                log_path = os.path.join(temp_folder, log_file)
                logs.append(log_path)
        return logs

    def run(self, test_case, test_path, parameters, queue):
        if "key" in parameters.keys():
            parameters.pop("key")
        print(parameters)
        command_ = self.generate_command(parameters)
        self.get_orig_logs()
        popen = subprocess.Popen(command_, shell=True)#stdout=subprocess.PIPE, stderr=subprocess.PIPE
        popen.communicate()
        ret_code = popen.returncode
        logs = self.get_test_log(test_case)
        print("TEST PLATFORM return code", ret_code)
        if ret_code == 0:
            test_result = State.PASS
        elif ret_code == 1:
            test_result = State.FAIL
        elif ret_code == 10:
            test_result = State.ERROR_UNHEALTHY
        else:
            test_result = State.BLOCK
        result = {"name": test_case, "result": test_result, "log": logs}
        print(result)
        queue.put(result)
        return ret_code




from test_framework.dut.sub_slot.oakgate_slot import OakgateSlot
from test_framework.dut.sub_slot.linux_slot import LinuxSlot
from test_framework.database.dut_database import DutDatabase
from utils.system import get_platform
from utils import log


class Slot(object):

    def __init__(self, agent):
        self.agent = agent
        self.agent_name = self.agent.get_agent_name()
        self.db_client = DutDatabase()
        self.platform = get_platform()

    def refresh(self):
        agent_id = self.db_client.get_agent_id(self.agent.ip_address, self.agent.port)
        if agent_id is not None:
            slots = self.db_client.get_agent_related_slots_by_id(agent_id)
            if self.platform == "oakgate":
                self.oakgate_refresh(slots, agent_id)
            else:
                self.linux_refresh(slots, agent_id)
        else:
            log.ERR("Slot is not find Agent: {}".format(";".join(self.agent.ip_address)))

    @staticmethod
    def oakgate_refresh(slots, agent_id):
        oak_slot = OakgateSlot(slots, agent_id)
        oak_slot.refresh_all()

    @staticmethod
    def linux_refresh(slots, agent_id):
        slot = LinuxSlot(slots, agent_id)
        slot.refresh_all()



#!/usr/bin/env python
# pylint: disable=superfluous-parens,missing-docstring,too-few-public-methods
"""
Created on 20/1, 2021

@author: wwang
"""
from ctypes import Structure, c_uint32


class HMBEntry(Structure):
    _pack_ = 1
    _fields_ = [
        ('UTI', c_uint32, 6),
        ('RSV0', c_uint32, 26),
        ('LS', c_uint32, 32),
        ('CHI', c_uint32, 32),
        ('CHO', c_uint32, 32),
        ('CTI', c_uint32, 32),
        ('CTO', c_uint32, 32),
        ('RSVD1', c_uint32, 32)
    ]
    desc = {
        'UTI': 'The Tenant ID that uses HMB',
        'LS': 'The lease size of this tenant',
        'CHI': 'Head index in HMB descriptor list entry',
        'CHO': 'Head offset to the start address of the head index',
        'CTI': 'Tail index in HMB descriptor list entry',
        'CT0': 'Tail offset to the start address of the tail index',
    }


class HMB(Structure):
    """HMB"""
    _pack_ = 1
    _fields_ = [
        ('Tenant id', HMBEntry * 14)  # Tenant Info for ID
    ]
    desc = {
        'Tenant id': 'Tenant Info for ID '
    }



#!/usr/bin/python
#-*-coding:utf-8-*-
import logging.handlers
import os
import sys
import time
LOGGER_STANDARD = None
LOGGER_DMESG = None


class Logger(object):
    """Class provides methods to perform logging."""
    m_logger = None
    log_path = None

    def __init__(self, opts, logfile, myname):
        """Set the default logging path."""
        self.opts = opts
        self.myname = myname
        self.console = None
        self.filelog = None
        self.filename = os.path.join('.', logfile)
        self.logfilename = os.path.normpath(os.path.expanduser(self.filename))
        Logger.log_path = self.logfilename

    @staticmethod
    def get_log_path():
        return Logger.log_path

    def loginit(self):
        """Calls function LoggerInit to start initialising the logging system."""
        self.logger_init(self.myname)

    def logger_init(self, loggername):
        """Initialise the logging system.
        This includes logging to console and a file. By default, console prints
        messages of level WARN and above and file prints level INFO and above.
        In DEBUG mode (-D command line option) prints messages of level DEBUG
        and above to both console and file.
        Args:
         loggername: String - Name of the application printed along with the log
         message.
        """
#         fileformat =
        # '%(asctime)s <%(filename)s %(funcName)s: %(lineno)d> %(levelname)-5s %(message)s'
        if loggername == "dmesg":
            fileformat = '%(asctime)s %(message)s'
        else:
            fileformat = '%(asctime)s <%(filename)10s: %(lineno)3d> %(levelname)-5s %(message)s'
        Logger.m_logger = logging.getLogger(loggername)
        Logger.m_logger.propagate = 0
        Logger.m_logger.setLevel(logging.INFO)
        self.console = logging.StreamHandler()
        self.console.setLevel(logging.CRITICAL)
        consformat = logging.Formatter(fileformat, datefmt="%H:%M:%S")
        self.console.setFormatter(consformat)
        self.filelog = logging.handlers.RotatingFileHandler(filename=self.logfilename,\
                                                             maxBytes=20*1024*1024, backupCount=5)
        self.filelog.setLevel(logging.INFO)
        self.filelog.setFormatter(consformat)
        Logger.m_logger.addHandler(self.filelog)
        if loggername != "dmesg":
            Logger.m_logger.addHandler(self.console)
        if self.opts['debug']:
            if 'LOGLEVEL' in os.environ:
                log_lever = os.environ['LOGLEVEL']
            else:
                log_lever = 'DEBUG'
            self.console.setLevel(log_lever)
            self.filelog.setLevel('DEBUG')
            Logger.m_logger.setLevel(log_lever)
        if not self.opts['nofork']:
            self.console.setLevel(logging.WARN)

    @staticmethod
    def log_stop():
        """Shutdown logging process."""
        logging.shutdown()

    def dump(self, stream, end='\n'):
        output = stream + end
        sys.stdout.write(output)
        sys.stdout.flush()


def default_log(log_tag):
    """define default log"""
    #debug mode & not in daemon
    opts = {'debug': True, 'nofork': True}
    iso_time_format = "%Y%m%d%H%M%S"
    log_dir = os.path.join(os.path.dirname(__file__),  "..", "log")
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)
    log_name = "%s_%s.log"%(log_tag, time.strftime(iso_time_format, time.localtime(time.time())))
    log_file = os.path.join(log_dir, log_name)
    log = Logger(opts, log_file, log_tag)
    log.loginit()
    return log, log.m_logger


Log, LOGGER_ = default_log("log")
# LOGGER_DMESG = default_log("dmesg")
DEBUG = LOGGER_.debug
INFO = LOGGER_.info
WARN = LOGGER_.warn
ERR = LOGGER_.error
DUMP = Log.dump




class State(object):
    NONE = -1
    FAIL = 0
    PASS = 1
    NOT_START = 2
    RUNNING = 3
    ABORT = 4

    BLOCK = 10
    ERROR_NOT_FOUND = 11
    ERROR_BASE_EXCEPTION = 12
    ERROR_TIMEOUT = 13
    ERROR_CONNECTION = 14
    ERROR_ABNORMAL_END = 15
    ERROR_UNHEALTHY = 16

    verdicts_map = {
        NONE: "NONE",
        FAIL: "FAIL",
        PASS: "PASS",
        NOT_START: "NOT_START",
        RUNNING: "RUNNING",
        ABORT: "ABORT",
        BLOCK: "BLOCK",
        ERROR_NOT_FOUND: "ERROR_NOT_FOUND",
        ERROR_BASE_EXCEPTION: "ERROR_BASE_EXCEPTION",
        ERROR_TIMEOUT: "ERROR_TIMEOUT",
        ERROR_CONNECTION: "ERROR_CONNECTION",
        ERROR_ABNORMAL_END: "ERROR_ABNORMAL_END",
        ERROR_UNHEALTHY: "ERROR_UNHEALTHY"
    }

    def __init__(self):
        pass


class TestType(object):

    TestCase = 1
    TestSuite = 2
    TestBenchmark = 3
    TestBenchmarkGroup = 4
    DownloadSerial = 10

    def __init__(self):
        pass


class AgentState(object):
    Online = 0
    Offline = 1
    Running = 2
    Idle = 3
    verdicts_map = {
        Online: "online",
        Offline: "offline",
        Running: "running",
        Idle: "idle",
    }


class SlotState(object):
    Idle = 0
    Standby = 1
    Build = 2
    Download = 3
    Testing = 4
    Lost = 5
    verdicts_map = {
        Idle: "Idle",
        Standby: "Standby",
        Build: "Build",
        Download: "Download",
        Testing: "Testing",
        Lost: "Lost",
    }


class DownloadType(object):

    NVMe = 0
    SerialTestPlatform = 1


#!/usr/bin/env python
# coding=utf-8

import os
from xml.dom.minidom import Document
from xml.dom.minidom import parse as parse
import xml.dom.minidom


class NoseXML(object):

    def __init__(self):

        impl = xml.dom.minidom.getDOMImplementation()
        self.dom = impl.createDocument(None, 'testsuite', None)
        self.root = self.dom.documentElement
        self._test = 0
        self._fail = 0
        self._pass = 0

    def _get_test_count(self, verdict):
        self._test = len(verdict)
        self._pass = len(filter(lambda n: n['test_result'] == Verdict.PASSED, verdict))
        self._fail = len(filter(lambda n: n['test_result'] == Verdict.FAILED, verdict))

    def _set_root_attr(self):
        self.root.setAttribute('name', 'nosetests')
        self.root.setAttribute('tests', str(self._test))
        self.root.setAttribute('failures', str(self._fail))
        self.root.setAttribute('pass', str(self._pass))

    def generate_xml(self, verdict):

        self._get_test_count(verdict)
        self._set_root_attr()
        for item in verdict:
            self._create_testcase_element(self.root, item)
        self.create_xml_file('nosetests.xml')

    def _create_testcase_element(self, parent_element, verdict):
        element = self.dom.createElement('testcase')
        element.setAttribute('classname', verdict['test_name'])
        element.setAttribute('name', verdict['test_name'])
        parent_element.appendChild(element)

        if verdict['test_result'] is not Verdict.PASSED:
            fail_element = self._create_failure_element(verdict)
            element.appendChild(fail_element)
        sys_out = self.dom.createElement('system-out')
        date = self.dom.createCDATASection(verdict['comment'])
        sys_out.appendChild(date)
        element.appendChild(sys_out)

    def _create_failure_element(self, verdict):
        fail_element = self.dom.createElement('failure')
        fail_element.setAttribute('type', 'exceptions. Error')
        fail_element.setAttribute('message', 'message: test case Error')
        date = self.dom.createCDATASection(verdict['comment'])
        fail_element.appendChild(date)
        return fail_element

    def create_xml_file(self, filename):
        with open(filename, 'w') as f:
            f.write(self.dom.toprettyxml(indent='\t', encoding='utf-8'))

    def parse_xml(self, xml_file):
        if xml_file is not None and os.path.exists(xml_file):
            DOMTree = parse(xml_file)
            Data = DOMTree.documentElement
            fail_out = self._get_failure_messge(Data)
            error_out = self._get_error_messge(Data)
            out_put = fail_out + error_out
            out_put = out_put.replace('\\n', '\n')
        else:
            out_put = ''
        return out_put

    def _get_failure_messge(self, root):
        nodes = root.getElementsByTagName('failure')
        out_put = ''
        for node in nodes:
            data = node.childNodes[0].data
            out_put = out_put + data
        return out_put

    def _get_error_messge(self, root):
        nodes = root.getElementsByTagName('error')
        out_put = ''
        for node in nodes:
            data = node.childNodes[0].data
            out_put = out_put + data
        return out_put


A = NoseXML()
B = A.parse_xml(r"D:\Automaiton\new_production\productionautomation\log\nosetests_test_rand_mix_rw_70_30_1583823250.6500623.xml")
print(B)



import os
import sys


class Parameters(object):

    def __init__(self):
        pass

    def pop_parm(self, parameters, key):
        if key in parameters.keys():
            parameters.pop(key)
        return parameters

    @staticmethod
    def get_default_base_path():
        if "win" in sys.platform.lower():
            base_path = r"\\172.29.190.4\share\sqa\FW_Release\redtail"   #shanghai r"\\10.3.10.2\Share\sqa\dell_bin\logs"
        else:
            base_path = "/mnt/sqa/FW_Release/redtail"
        return base_path

    def generate_redtail_images(self, parameters):
        output_parm = dict()
        if "base_path" in parameters:
            base_path = parameters["base_path"]
        else:
            base_path = self.get_default_base_path()
        if "image1" not in parameters.keys() and "volume" in parameters.keys() and "base_version" in parameters.keys():
            ret = self.get_image_path(base_path, parameters["volume"], parameters["base_version"])
            if ret is not None:
                output_parm["image1"] = ret
                output_parm["base_version"] = parameters["base_version"]
        if "image2" not in parameters.keys():
            if "volume" in parameters.keys() and "target_version" in parameters.keys():
                ret = self.get_image_path(base_path, parameters["volume"], parameters["target_version"])
                if ret is not None:
                    output_parm["image2"] = ret
                    output_parm["target_version"] = parameters["target_version"]
            else:
                if "image1" in output_parm.keys():
                    output_parm["image2"] = output_parm["image1"]
                    output_parm["target_version"] = output_parm["base_version"]
        return output_parm

    def get_image_path(self, base_path, volume, commit_id):
        # commit_id = commit_id[0:6]
        _files = os.listdir(base_path)
        if os.path.exists(base_path):
            for item in _files:
                item_path = os.path.join(base_path, item)
                if os.path.isdir(item_path):
                    ret = self.get_image_path(item_path, volume, commit_id)
                    if ret is not None:
                        return ret
                elif ("_{}_".format(volume) in item) and (commit_id in item):
                    return os.path.join(base_path, item)



import sys
import re
if "win" not in sys.platform.lower():
    from tool.device.nvme.nvme import Tahoe
from utils.system import execute


class LinuxNvme(object):

    def __init__(self):
        pass

    @staticmethod
    def get_info(cntid=0):
        driver = Tahoe(cntid=cntid)
        information = driver.get_info()
        return information

    @staticmethod
    def get_linux_nvme_devs():
        dev_list = list()
        control_list = list()
        cmd = "ls /dev/n*"
        _, outs = execute(cmd, console=False)
        rets = re.findall("/dev/(nvme(\d+))\s", outs, re.DOTALL)
        for item in rets:
            if item[1] not in control_list:
                dev = {"ctrl_id": item[1], "name": item[0]}
                control_list.append(item[1])
                dev_list.append(dev)
        return dev_list



# coding=utf-8
import os
import  xml.dom.minidom
from utils.system import get_ip_address

class TestResult(object):


    def __init__(self):
        pass

    def _get_log_txt(self, log_path):
        msg = "Log path: {}".format(log_path)
        if os.path.exists(log_path):
            file_size = os.path.getsize(log_path)
            if file_size > 100*1024*1024:
                txt_ = "Log file more than 100M, please check it on test environment: {}".format(log_path)
            else:
                txt_ = self.read_log(log_path)
        else:
            txt_ = "Log file not exists: {}".format(log_path)
        msg = "{} \n{}".format(msg, txt_)
        return msg

    @staticmethod
    def read_log(log_path):
        file_ = open(log_path, encoding='utf-8')
        lines_ = file_.readlines()
        numbers = len(lines_)
        if numbers > 1000:
            new_lines = lines_[0:30]
            new_lines.append("............\n............\n............\n")
            end_lines = lines_[-970:]
            new_lines.extend(end_lines)
        else:
            new_lines = lines_
        logs = ''.join(new_lines)
        file_.close()
        return logs

    def get_xml_report_txt(self, xml_path):
        log_path_print = "Log at: {} \n".format(xml_path)
        dom = xml.dom.minidom.parse(xml_path)
        root = dom.documentElement
        system_out_element = root.getElementsByTagName('system-out')
        system_out_tex = system_out_element[0].firstChild.wholeText if system_out_element else ""
        fail_element = root.getElementsByTagName('failure')
        fail_txt = fail_element[0].firstChild.wholeText if fail_element else ""
        error_element = root.getElementsByTagName('error')
        error_txt = error_element[0].firstChild.wholeText if error_element else ""
        sys_error_element = root.getElementsByTagName('system-err')
        sys_error_txt = sys_error_element[0].firstChild.wholeText if sys_error_element else ""
        xml_logs = log_path_print + system_out_tex + fail_txt + error_txt + sys_error_txt

        return xml_logs

    def get_log(self, logs):
        log = str()
        try:
            if type(logs) is list:
                for item in logs:
                    log_content = self._get_log_txt(item)
                    log = "{} \n\n{}\n\n{}".format(log, item, log_content)
            else:
                log = self._get_log_txt(logs)
        except Exception as all_exception:
            print(all_exception)
        return log

    def get_test_suite_test_msg(self, test_results):
        msg = ""
        if test_results is not None:
            for test_result in test_results:
                if "xml_path" in test_result.keys():
                    xml_log = self.get_xml_logs(test_result["xml_path"])
                    msg = msg + "\n \n ****************************************** \n \n"
                    msg = msg + test_result["name"] + "\n"
                    msg = msg + "\n" + xml_log
                elif "msg" in test_result.keys():
                    msg = msg + str(test_result["msg"])
                elif "log" in test_result.keys():
                    logs = test_result["log"]
                    msg = msg + str(self.get_log(logs))
        return msg

    @staticmethod
    def check_file_size(path):
        result = False
        if os.path.exists(path):
            file_size = os.path.getsize(path)
            result = True if file_size < 10*1024*1024 else False
        return result

    def get_xml_logs(self, log_path):
        if os.path.exists(log_path):
            if self.check_file_size(log_path):
                xml_log = self.get_xml_report_txt(log_path)
            else:
                xml_log = "log file more than 10M, do not upload file.\n" \
                          "logs at host: {}, path: {}".format(get_ip_address(), log_path)
        else:
            xml_log = "log file not exist, test abnormal end, it may test case script issue, please check test case\n" \
                      "logs at host: {}, path: {}".format(get_ip_address(), log_path)
        return xml_log



# coding=utf-8
# pylint: disable=broad-except, import-error
import os
import yaml
from datetime import datetime, timezone, timedelta
import threading
import pymysql as mysql
from utils import log
from test_framework.state import State
from utils.system import get_ip_address


class SqlConnection(object):

    def __init__(self, db_name="dutdb"):
        self.host = None
        self.port = None
        self.user = None
        self.passwd = None
        self.db_name = db_name
        self.get_data_base_config()
        self.conn = mysql.connect(host=self.host, port=self.port, user=self.user,
                                  passwd=self.passwd, db=self.db_name, charset="utf8")
        self.cursor = self.conn.cursor()

    def get_data_base_config(self):
        config_file = os.path.join(os.path.dirname(__file__), "../..", "configuration", "database.yaml")
        with open(config_file, encoding='utf-8') as f:
            db_config = yaml.safe_load(f)
            self.host = db_config["mysql"]["host"]
            self.port = db_config["mysql"]["port"]
            self.user = db_config["mysql"]["user"]
            self.passwd = db_config["mysql"]["passwd"]

    def reconnect(self):
        self.conn = mysql.connect(host=self.host, port=self.port, user=self.user,
                                  passwd=self.passwd, db=self.db_name, charset="utf8")
        self.cursor = self.conn.cursor()

    def __del__(self):
        self.cursor.close()
        self.conn.close()

    def ping(self):
        return self.conn.ping()

    def get_datetime(self):
        tz = timezone(timedelta(hours=+8))
        current_time = datetime.now(tz).strftime("%Y-%m-%d %H:%M:%S")
        return current_time

    def insert_to_table(self, table, **kwargs):
        col_str, value_str = self._covert_dict_2_insert_string(**kwargs)
        insert_command = "INSERT INTO `{}` ({}) VALUES({})".format(table, col_str, value_str)
        self.cursor.execute(insert_command)
        self.conn.commit()

    @staticmethod
    def _covert_dict_2_insert_string(**kwargs):
        col_str = ""
        value_str = ""
        for key, value in kwargs.items():
            col_str = "`{}`".format(key) if col_str == "" else "{},`{}`".format(col_str, key)
            value_str = "'{}'".format(value) if value_str == "" else "{},'{}'".format(value_str, value)
        return col_str, value_str

    @staticmethod
    def _covert_dict_2_update_string(**kwargs):
        update_str = ""
        for key, value in kwargs.items():
            temp_str = "`{}`='{}'".format(key, value)
            update_str = "{},{}".format(update_str, temp_str) if update_str != "" else temp_str
        return update_str

    def insert_test_result(self, **kwargs):
        self.insert_to_table("test_results", **kwargs)

    def exist_key(self, test_key):
        sql_command = "SELECT test_key from test_results WHERE test_key='{}'".format(test_key)
        self.cursor.execute(sql_command)
        result = self.cursor.fetchone()
        return result

    def update_test_states(self, test_key, **kwargs):
        str_date = ""
        for key, value in kwargs.items():
            temp_str = "{}='{}'".format(key, value)
            str_date = temp_str if str_date == "" else "{},{}".format(str_date, temp_str)
            if key == "result":
                if value == 3:
                    temp_str = "{}='{}'".format("start_time", self.get_datetime())
                    str_date = "{},{}".format(str_date, temp_str)
                elif value in [0, 1]:
                    temp_str = "{}='{}'".format("end_time", self.get_datetime())
                    str_date = "{},{}".format(str_date, temp_str)
        update_command = "UPDATE test_results SET {} where test_key='{}'".format(str_date, test_key)
        self.cursor.execute(update_command)
        self.conn.commit()

    def update_test_to_abnormal_end(self, ip_addr):
        update_command = "UPDATE test_results SET result='15' where ip='{}' AND (result='3' OR result='2')".format(ip_addr)
        self.cursor.execute(update_command)
        self.conn.commit()

    def create_table(self, table_name, col_attrs):
        str_col = None
        for item in list(col_attrs):
            if str_col is None:
                str_col = "`{}` {}".format(item["name"], item["type"])
            else:
                str_col = "{}, `{}` {}".format(str_col, item["name"], item["type"])
        command = "CREATE TABLE `{}` ({})".format(table_name, str_col)
        self.cursor.execute(command)
        self.conn.commit()

    def get_last_index(self, table_name='tests'):
        command = "select `index` from {} ORDER by `index` desc".format(table_name)
        self.cursor.execute(command)
        result = self.cursor.fetchone()
        index = result[0] if result else 0
        return index

    def is_exist_table(self, table):
        sql_command = "select * from information_schema.TABLES  where TABLE_NAME='{}';".format(table)
        self.cursor.execute(sql_command)
        gets = self.cursor.fetchone()
        result = True if gets else False
        return result

    def execute_sql_command(self, command):
        results = list()
        try:
            self.conn.ping()
            self.cursor.execute(command)
            results = self.cursor.fetchall()
        except BaseException as message:
            print(message)
        return results


def decorate_add_tests(func):
    def func_wrapper(*args, **kwargs):
        ret = func(*args, **kwargs)
        if ret is not None:
            SQL = SqlConnection()
            for index in range(10):
                log.INFO("decorate_add_tests {} loop {}".format(args[1], index))
                try:
                    SQL.ping()
                    SQL.insert_test_result(test_key=ret, test_name=args[1], result=State.NOT_START, ip=get_ip_address())
                    break
                except Exception as all_exception:
                    log.ERR(all_exception)
        return ret
    return func_wrapper


def decorate_update_test_state(func):
    def func_wrapper(*args, **kwargs):
        ret = func(*args, **kwargs)
        logs = "not record logs"
        SQL = SqlConnection()
        for index in range(10):
            log.INFO("decorate_update_test_state loop {}".format(index))
            try:
                SQL.ping()
                SQL.update_test_states(args[1], result=args[2], test_log=logs)
                break
            except Exception as all_exception:
                log.ERR(all_exception)
        return ret
    return func_wrapper



#!/usr/bin/env python
"""
Created on Apr 17, 2017

@author: yyang
"""
from ctypes import Structure, c_uint8, c_uint32, c_uint64, Union
from tool.device.nvme.ctype import Uint128


class CDW(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('BYTE', c_uint8 * 4),
        ]

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]


class Arbitration(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('AB', c_uint32, 3),  # Arbitration Burst (AB)
            ('RSV', c_uint32, 5),
            ('LPW', c_uint32, 8),  # Low Priority Weight (LPW)
            ('MPW', c_uint32, 8),  # Medium Priority Weight (MPW)
            ('HPW', c_uint32, 8),  # High Priority Weight (HPW)
        ]
        desc = {
            'AB': 'Arbitration Burst', 'LPW': 'Low Priority Weight',
            'MPW': 'Medium Priority Weight', 'HPW': 'High Priority Weight'
        }

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class PowerManagement(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('PS', c_uint32, 5),  # Power State
            ('WH', c_uint32, 3),  # Workload Hint
            ('RSV', c_uint32, 24),
        ]
        desc = {'PS': 'Power State', 'WH': 'Workload Hint'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class LBARangeType(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('NUM', c_uint32, 6),  # Number of LBA Ranges
            ('RSV', c_uint32, 24),
        ]
        desc = {'NUM': 'Number of LBA Ranges'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class LBARangeTypeEntry(Structure):
    _pack_ = 1
    _fields_ = [
        ('TYPE', c_uint8),  # Type
        ('ATTR', c_uint8),  # Attributes
        ('RSV0', c_uint8 * 14),
        ('SLBA', c_uint64),  # Starting LBA
        ('NLB', c_uint64),  # Number of Logical Blocks
        ('GUID', Uint128),  # Unique Identifier
        ('RSV1', c_uint8 * 16),
    ]
    desc = {
        'TYPE': 'Type', 'ATTR': 'Attributes', 'SLBA': 'Starting LBA', 'NLB': 'Number of Logical Blocks',
        'GUID': 'Unique Identifier'
    }


class TemperatureThreshold(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('TMPTH', c_uint32, 16),  # Temperature Threshold
            ('TMPSEL', c_uint32, 4),  # Threshold Temperature Select
            ('THSEL', c_uint32, 2),  # Threshold Type Select
            ('RSV', c_uint32, 10),
        ]
        desc = {'TMPTH': 'Temperature Threshold', 'TMPSEL': 'Threshold Temperature Select',
                'THSEL': 'Threshold Type Select'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class ErrorRecovery(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('TLER', c_uint32, 16),  # Time Limited Error Recovery:
            ('DULBE', c_uint32, 1),  # Deallocated or Unwritten Logical Block Error Enable
            ('RSV', c_uint32, 15),
        ]
        desc = {
            'TLER': 'Time Limited Error Recovery:', 'DULBE': 'Deallocated or Unwritten Logical Block Error Enable'
        }

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class VolatileWriteCache(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('WCE', c_uint32, 1),  # Volatile Write Cache Enable
            ('RSV', c_uint32, 31),
        ]
        desc = {'WCE': 'Volatile Write Cache Enable'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class NumberOfQueues(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('NSQR', c_uint32, 16),  # Number of I/O Submission Queues Requested (NSQR)
            ('NCQR', c_uint32, 16),  # Number of I/O Completion Queues Requested (NCQR)
        ]
        desc = {
            'NSQR': 'Number of I/O Submission Queues Requested',
            'NCQR': 'Number of I/O Completion Queues Requested'
        }

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class NumberOfQueuesEntry(Structure):
    _pack_ = 1
    _fields_ = [
        ('NSQA', c_uint32, 16),  # Number of I/O Submission Queues Allocated (NSQA)
        ('NCQA', c_uint32, 16),  # Number of I/O Completion Queues Allocated (NCQA)
    ]
    desc = {
        'NSQA': 'Number of I/O Submission Queues Allocated',
        'NCQA': 'Number of I/O Completion Queues Allocated'
    }


class InterruptCoalescing(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('THR', c_uint32, 8),  # Aggregation Threshold
            ('TIME', c_uint32, 8),  # Aggregation Time
            ('RSV0', c_uint32, 16),
        ]
        desc = {'THR': 'Aggregation Threshold', 'TIME': 'Aggregation Time'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class InterruptVectorConfiguration(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('IV', c_uint32, 16),  # Interrupt Vector (IV)
            ('CD', c_uint32, 1),  # Coalescing Disable (CD)
            ('RSV', c_uint32, 15),
        ]
        desc = {'IV': 'Interrupt Vector', 'CD': 'Coalescing Disable'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class WriteAtomicityNormal(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('DN', c_uint32, 1),  # Disable Normal
            ('RSV', c_uint32, 31),
        ]
        desc = {'DN': 'Disable Normal'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class AsynchronousEventConfiguration(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('SHCW', c_uint32, 8),  # SMART / Health Critical Warnings (SHCW)
            ('NAN', c_uint32, 1),  # Namespace Attribute Notices (NAN)
            ('FAN', c_uint32, 1),  # Firmware Activation Notices (FAN)
            ('TLN', c_uint32, 1),  # Telemetry Log Notices (TLN)
            ('RSV', c_uint32, 22),
        ]
        desc = {
            'SHCW': 'SMART / Health Critical Warnings', 'NAN': 'Namespace Attribute Notices',
            'FAN': 'Firmware Activation Notices', 'TLN': 'Telemetry Log Notices'
        }

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class AutonomousPowerStateTransition(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('APSTE', c_uint32, 1),  # Autonomous Power State Transition Enable
            ('RSV', c_uint32, 31),
        ]
        desc = {'APSTE': 'Autonomous Power State Transition Enable'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class AutonomousPowerStateTransitionEntry(Structure):
    _pack_ = 1
    _fields_ = [
        ('RSV0', c_uint32, 3),
        ('ITPS', c_uint32, 5),  # Idle Transition Power State
        ('ITPT', c_uint32, 24),  # Idle Time Prior to Transition
        ('RSV1', c_uint32, 32),
    ]
    desc = {'ITPS': 'Idle Transition Power State', 'ITPT': 'Idle Time Prior to Transition'}


class HostMemoryBuffer(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('EHM', c_uint32, 1),  # Enable Host Memory (EHM)
            ('MR', c_uint32, 1),  # Memory Return (MR)
            ('RSV', c_uint32, 30),
        ]
        desc = {'EHM': 'Enable Host Memory', 'MR': 'Memory Return'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class HostMemoryBufferDescriptorEntry(Structure):
    _pack_ = 1
    _fields_ = [
        ('BADD', c_uint64),  # Buffer Address (BADD)
        ('BSIZE', c_uint32),  # Buffer Size (BSIZE)
        ('RSV', c_uint32),
    ]
    desc = {'BADD': 'Buffer Address', 'BSIZE': 'Buffer Size'}


class HostMemoryBufferDescriptorList(Structure):
    _pack_ = 1
    _fields_ = [
        ('ENTRY', HostMemoryBufferDescriptorEntry * 256),  # Host Memory Buffer Descriptor Entry
    ]
    desc = {'ENTRY': 'Host Memory Buffer Descriptor Entry'}


class HostMemoryBufferDataStructure(Structure):
    _pack_ = 1
    _fields_ = [
        ('HSIZE', c_uint32),  # Host Memory Buffer Size (HSIZE)
        ('HMDLAL', c_uint32),  # Host Memory Descriptor List Address Lower (HMDLAL)
        ('HMDLAU', c_uint32),  # Host Memory Descriptor List Address Upper (HMDLAU)
        ('HMDLEC', c_uint32),  # Host Memory Descriptor List Entry Count (HMDLEC)
        ('RSV', c_uint64 * 510),
    ]
    desc = {
        'HSIZE': 'Host Memory Buffer Size',
        'HMDLAL': 'Host Memory Descriptor List Address Lower',
        'HMDLAU': 'Host Memory Descriptor List Address Upper',
        'HMDLEC': 'Host Memory Descriptor List Entry Count'
    }


class Timestamp(Structure):
    _pack_ = 1
    _fields_ = [
        ('TIMESTAMP', c_uint64, 48),  # Number of milliseconds that have elapsed since midnight, 01-Jan-1970, UTC.
        ('SYNCH', c_uint64, 1),
        ('TO', c_uint64, 3),  # Timestamp Origin (TO)
        ('RSV0', c_uint64, 4),
        ('RSV1', c_uint64, 8),
    ]
    desc = {'TIMESTAMP': 'Number of milliseconds since 01-Jan-1970, UTC', 'TO': 'Timestamp Origin'}


class KeepAliveTimer(Structure):
    _pack_ = 1
    _fields_ = [
        ('KATO', c_uint32),  # Persist Through Power Loss (KATO)
    ]
    desc = {'KATO': 'Persist Through Power Loss'}


class HostControlledThermalManagement(Structure):
    _pack_ = 1
    _fields_ = [
        ('TMT2', c_uint32, 16),  # Thermal Management Temperature (TMT)
        ('TMT1', c_uint32, 16),
    ]
    desc = {'TMT2': 'Thermal Management Temperature 2', 'TMT1': 'Thermal Management Temperature 1'}


class NonOperationalPowerStateConfig(Structure):
    _pack_ = 1
    _fields_ = [
        ('NOPPME', c_uint32, 1),  # Non-Operational Power State Permissive Mode Enable (NOPPME)
        ('RSV', c_uint32, 31),
    ]
    desc = {'NOPPME': 'Non-Operational Power State Permissive Mode Enable'}


class ReadRecoveryLevelConfig(Structure):
    _pack_ = 1
    _fields_ = [
        ('RRL', c_uint32, 4),
        ('RSV0', c_uint32, 28),
    ]


class SanitizeConfig(Structure):
    _pack_ = 1
    _fields_ = [
        ('NODRM', c_uint32, 1),
        ('RSV0', c_uint32, 31),
    ]


class SoftwareProgressMarker(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('PBSLC', c_uint32, 8),
            ('RSV', c_uint32, 24),
        ]
        desc = {'PBSLC': 'Pre-boot Software Load Count'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class HostIdentifier(Union):
    class BITS(Structure):
        _pack_ = 1
        _fields_ = [
            ('EXHID', c_uint32, 1), # Enable Extended Host Identifier
            ('RSV', c_uint32, 31),
        ]
        desc = {'EXHID': 'Enable Extended Host Identifier'}

    _anonymous_ = ('BITS',)
    _fields_ = [
        ('DWORD', c_uint32),
        ('BITS', BITS),
    ]
    desc = {'DWORD': 'Command Dword'}


class HostIdentifierEntry(Structure):
    _pack_ = 1
    _fields_ = [
        ('HOSTIDL', c_uint64),
        ('HOSTIDH', c_uint64),  # HOSTID
    ]
    desc = {
        'HOSTIDL': 'Host Identifie',
        'HOSTIDH': 'Host Identifie'
    }


class ReservationNotificationMask(Structure):
    _pack_ = 1
    _fields_ = [
        ('RSV0', c_uint32, 1),
        ('REGPRE', c_uint32, 1),  # Mask Registration Preempted Notification
        ('RESREL', c_uint32, 1),  # Mask Reservation Released Notification
        ('RESPRE', c_uint32, 1),  # Mask Reservation Preempted Notification
        ('RSV1', c_uint32, 28),
    ]
    desc = {
        'REGPRE': 'Mask Registration Preempted Notification', 'RESREL': 'Mask Reservation Released Notification',
        'RESPRE': 'Mask Reservation Preempted Notification'
    }


class ReservationPersistence(Structure):
    _pack_ = 1
    _fields_ = [
        ('PTPL', c_uint32, 1),  # Persist Through Power Loss (PTPL)
        ('RSV0', c_uint32, 31),
    ]
    desc = {'PTPL': 'Persist Through Power Loss'}


class FeatureID(object):
    Arbitration = 0x1
    PowerManagement = 0x2
    LBARangeType = 0x3
    TemperatureThreshold = 0x4
    ErrorRecovery = 0x5
    VolatileWriteCache = 0x6
    NumberOfQueues = 0x7
    InterruptCoalescing = 0x8
    InterruptVectorConfiguration = 0x9
    WriteAtomicityNormal = 0xa
    AsynchronousEventConfiguration = 0xb
    AutonomousPowerStateTransition = 0xc
    HostMemoryBuffer = 0xd
    Timestamp = 0xe
    KeepAliveTimer = 0xf
    HostControlledThermalManagement = 0x10
    NonOperationalPowerStateConfig = 0x11
    ReadRecoveryLevelConfig = 0x12
    PredictableLatencyModeConfig = 0x13
    PredictableLatencyModeWindow = 0x14
    LBAStatusInfoReportInterval = 0x15
    HostBehaviorSupport = 0x16
    SanitizeConfig = 0x17
    EnduranceGroupEventConfig = 0x18
    SoftwareProgressMarker = 0x80
    HostIdentifier = 0x81
    ReservationNotificationMask = 0x82
    ReservationPersistence = 0x83
    NamespaceWriteProtectionConfig = 0x84


GET_FEATURE_ID = {
    0x1: Arbitration,
    0x2: PowerManagement,
    0x3: LBARangeType,
    0x4: TemperatureThreshold,
    0x5: ErrorRecovery,
    0x6: VolatileWriteCache,
    0x7: NumberOfQueues,
    0x8: InterruptCoalescing,
    0x9: InterruptVectorConfiguration,
    0xa: WriteAtomicityNormal,
    0xb: AsynchronousEventConfiguration,
    0xc: AutonomousPowerStateTransition,
    0xd: HostMemoryBuffer,
    0xf: KeepAliveTimer,
    0x10: HostControlledThermalManagement,
    0x11: NonOperationalPowerStateConfig,
    0x80: SoftwareProgressMarker,
    0x81: HostIdentifier,
    0x82: ReservationNotificationMask,
    0x83: ReservationPersistence,
}

GET_FEATURE_ENTRY = {
    0x3: LBARangeTypeEntry,
    0xd: HostMemoryBufferDataStructure,
    0xe: Timestamp,
}



#!/usr/bin/env python
# coding=utf-8
import os
import sys
import yaml
import argparse
sys.path.append(os.path.join(os.path.dirname(__file__)))
from rest_server.reset_server import *
from rest_server.resource.models.ftp_server import thread_start_ftp_server
from test_framework.test_pool import TestPool
from utils.excel import ExcelHandle
from test_framework.database.dut_database import DutDatabase


def add_sub_argument_group(subparsers, name, handler_function):
    regression_parser = subparsers.add_parser(name, help='%s tests executor'%name)
    regression_required_arguments = regression_parser.add_argument_group('required arguments')
    regression_required_arguments.add_argument('--port', '-p', type=str, default="5000",
                                               help='start server port id', required=False)
    regression_required_arguments.add_argument('--work_path', '-w', type=str, default="",
                                               help='automation platform path, test-platform or perses', required=False)
    regression_required_arguments.add_argument('--platform', '-pl', type=str, default="",
                                               help='host platform: oakgate or linux', required=False)
    regression_required_arguments.add_argument('--excel', '-e', type=str, default="",
                                               help='oakgate config excel file', required=False)
    regression_parser.set_defaults(executor_function=handler_function)


def create_parser():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()
    add_sub_argument_group(subparsers, 'start', start_rest_server)
    add_sub_argument_group(subparsers, 'db', update_db)
    return parser


def update_db(args):
    dut_db = DutDatabase()
    configs = ExcelHandle.get_ogt_config(args.excel)
    dut_db.update_slot_agent_2_id()
    dut_db.set_ogt_config_2_slot(configs)


def start_rest_server(args):
    _ = TestPool()
    thread_start_ftp_server()
    APP.run(host="0.0.0.0", port=args.port)


def load_global_config():
    config_file = os.path.join(os.getcwd(), 'config.yml')
    if os.path.exists(config_file):
        conf = yaml.load(open(config_file).read(), Loader=yaml.FullLoader)
        for key, val in conf.items():
            os.environ[key] = val


def get_work_path(args):
    work_path = os.getcwd()
    if args.work_path != "":
        if os.path.exists(args.work_path):
            work_path = args.work_path
    return work_path


def add_globals(args):
    load_global_config()
    os.environ["root_path"] = os.path.join(os.path.dirname(__file__))
    os.environ["working_path"] = get_work_path(args)
    os.environ["agent_port"] = args.port
    os.environ["platform"] = args.platform


def run():
    parser = create_parser()
    args = parser.parse_args()
    try:
        add_globals(args)
        args.executor_function(args)
    except Exception as e:
        print(e)


if __name__ == '__main__':
    run()



from setuptools import setup, find_packages
packages = ["web_agent"]
#python setup.py sdist upload  
#python setup.py bdist_wheel

setup(
    name = 'web_agent',
    version = '1.1.8',
    keywords = ['runner', 'server'],
    description = 'NVMe production server',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
      ],
    license = 'MIT License',
	url = 'https://pypi.org/project/web_agent',
    install_requires = ['flask',
						'flask-restful',
						'requests',
						'PyMySQL==1.0.2',
						'pyftpdlib',
						'nose',
						'nose-printlog',
                        'checksumdir',
                        'pyyaml'],
    packages = find_packages(),
    include_package_data=True, 
    author = 'yuwen123441',
    author_email = 'yuwen123441@126.com',
    platforms = 'any',
    entry_points = {
        'console_scripts': [
        'agent = web_agent.run:run'
        ]}
)


# coding=utf-8
# pylint: disable=unused-variable
import os
import time
import yaml
from test_framework.test_base import TestBase
from utils import log
from test_framework.test_runner import Runner


class TestSuite(TestBase):

    def __init__(self):
        super(TestSuite, self).__init__()
        self.runner = Runner()
        self.test_suites = list()
        self.working_path =  os.environ["working_path"]
        self.test_suite_folder = self.get_test_suite_path()
        self.get_all_test_suites(self.test_suite_folder)

    def get_test_suite_path(self):
        test_suite_path = None
        dirs = os.listdir(self.working_path)
        for item in dirs:
            temp_path = os.path.join(self.working_path, item)
            if os.path.isdir(temp_path) and "suite" in item.lower():
                    test_suite_path = temp_path
        return test_suite_path

    def load_test_suite(self, test_suite):
        test_suite = test_suite+".yaml" if ".yaml" not in test_suite else test_suite
        test_suite_path = self.get_file_path(self.test_suite_folder, test_suite)
        if test_suite_path is not None and os.path.exists(test_suite_path):
            file_ = open(test_suite_path)
            test_suites = yaml.load(file_)
        else:
            raise Exception("TestSuite not exist: {}".format(test_suite))
        return test_suites

    def stop(self):
        return self.runner.stop()

    def production_run(self, test_suite):
        try:
            test_suites = self.load_test_suite(test_suite)
        except BaseException as message:
            print(message)
            result = [{"name": test_suite, "result": False}]
            return result
        for loop in range(int(test_suites["loop"])):
            log.INFO("Test Suite:%s, loop:%d", test_suites["name"], loop)
            for tests in test_suites["cases"]:
                test_path = self.get_all_script_path(tests["script"])
                if not test_path:
                    log.ERR("TestCase not find: %s", tests["script"])
                    break
                result = self.runner.process_run(tests["script"], test_case_path=test_path[0],
                                                 loop=tests["loop"], timeout=tests["duration"])
                if result["result"] is False:
                    break
                time.sleep(3)
        self.print_results(test_suite)
        return self.runner.get_results()

    def oakgate_run(self, test_suite, test_parameters):
        self.runner.process_run(test_suite, None, test_parameters)
        return self.runner.get_results()

    def run(self, test_suite, test_parameters=None):
        if "test-platform" in self.working_path:
            result = self.oakgate_run(test_suite, test_parameters)
        else:
            result = self.production_run(test_suite)
        return result

    def get_test_suites(self):
        return self.test_suites

    def list_and_filter_tests(self, filters):
        if filters.lower() == "all":
            test_suites = self.test_suites
        else:
            test_suites = [test_suit for test_suit in self.test_suites if "filters" in test_suit]
        return test_suites

    def get_all_test_suites(self, path):
        if path is None or os.path.exists(path) is False:
            return
        _files = os.listdir(path)
        for item in _files:
            item_path = os.path.join(path, item)
            if os.path.isdir(item_path):
                ret = self.get_all_test_suites(item_path)
                if ret is not None:
                    return ret
            elif ".yaml" in item:
                item = os.path.basename(item_path)
                self.test_suites.append(item)


    def print_results(self, test_suite):
        result = self.runner.get_results()
        log.INFO("###########################################################")
        log.INFO("********************* Result Summary **********************")
        log.INFO("********************* TestSuite: %s ***********************", test_suite)
        pass_items = filter(lambda x: x["result"] is True, result)
        fail_items = filter(lambda x: x["result"] is False, result)
        for item in result:
            if item["result"] is True:
                log.INFO("TestCase: %s, result: PASS", item["name"])
            else:
                log.ERR("TestCase: %s, result: FAIL", item["name"])
        log.INFO("Total run %s, pass %s, fail %s", len(result), len(list(pass_items)), len(list(fail_items)))
        log.INFO("###########################################################")



# coding=utf-8
# pylint: disable=invalid-name
import threading
import ctypes
import inspect
from flask_restful import fields


resource_fields = {
    'data': fields.List(fields.String, default=None),
    'msg': fields.String,
    'state': fields.Integer
}


class MyThread(threading.Thread):

    def __init__(self, group=None, target=None, name=None,
                 args=(), kwargs=None, daemon=None):
        super(MyThread, self).__init__()
        self.args = args
        self.target = target
        self.result = None
        self._stop_event = threading.Event()

    def run(self):
        self.result = self.target(*self.args)
        print(self.result)

    def get_result(self):
        try:
            return list(self.result)
        except BaseException:
            return None

    def _async_raise(self, tid, exctype):
        """raises the exception, performs cleanup if needed"""
        tid = ctypes.c_long(tid)
        if not inspect.isclass(exctype):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        # if res == 0:
        #     raise ValueError("invalid thread id")
        if res != 1:
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")

    def stop(self):
        # self._stop_event.set()
        self._async_raise(self.ident, SystemExit)

    def stopped(self):
        return self._stop_event.is_set()




import os
import platform
from utils.system import get_multi_ip_address, get_platform, get_host_name
from test_framework.database.dut_database import update_agent


class Agent(object):

    def __init__(self):
        self.ip_address = get_multi_ip_address()
        self.port = os.environ.get('agent_port', '5000')

    def get_agent_info(self):
        agent_info = dict()
        agent_info["name"] = get_host_name()
        agent_info["ip"] = ";".join(self.ip_address)
        agent_info["port"] = self.port
        agent_info["os"] = "windows" if platform.system() == 'Windows' else "linux"
        agent_info["platform"] = get_platform()
        return agent_info

    @staticmethod
    def get_agent_name():
        return get_host_name()

    @update_agent
    def refresh(self):
        return self.get_agent_info()


if __name__ == '__main__':
    ag = Agent()
    ag.refresh()




import os
import yaml
from utils import log
from test_framework.database.dut_database import update_slot
from test_framework.state import SlotState
from utils.system import get_vendor_name


class OakgateSlot(object):

    def __init__(self, slots, agent_id):
        self.db_slots = slots
        self.agent_id = agent_id
        self.orig_log_folders = list()
        self.platform_path = os.environ.get('working_path')
        self.logs_path = os.path.join(self.platform_path, "Logs")
        self.script = "u_ssd_data_collection"

    def refresh_all(self):
        for slot in self.db_slots:
            slot_id = slot[0]
            config_name = slot[2]
            self.refresh(slot_id, config_name)

    def execute_command(self, config_name):
        command_line = "cd /d {} && py -3 run.py --oakgate {} --script {}.py".\
            format(self.platform_path, config_name, self.script)
        log.INFO("Oakgate execute command: {}".format(command_line))
        return os.system(command_line)

    def get_orig_logs(self):
        all_logs = os.listdir(self.logs_path)
        self.orig_log_folders = [item for item in all_logs if self.script in item]

    def get_new_logs(self):
        latest_log_folders = os.listdir(self.logs_path)
        new_logs = list()
        for item in latest_log_folders:
            if item not in self.orig_log_folders:
                if os.path.isdir(os.path.join(self.logs_path, item)):
                    if self.script in item:
                        new_logs.append(item)
        return new_logs

    def load_result(self):
        format_result = dict()
        logs = self.get_new_logs()
        if logs:
            files = os.listdir(os.path.join(self.logs_path, logs[0]))
            for item in files:
                if "SSD_data.yaml" in item:
                    log.INFO("Find command output yaml: {}".format(item))
                    with open(os.path.join(self.logs_path, logs[0], item), encoding='utf-8') as f:
                        results = yaml.safe_load(f)
                        format_result = self._format_results(results)
        log.INFO("get result:")
        print(format_result)
        return format_result

    def _format_results(self, results):
        format_result = {
            "slot": results["oakgate_config_dic"]["oakgate_port_name"],
            "vendor": get_vendor_name(results["ssd_config_dic"]["drive_vid"]),
            "fw_version": results["test_fw_config_dic"]["fw_public_revision"],
            "commit": results["test_fw_config_dic"]["fw_private_revision"],
            "ise/sed": results["ssd_config_dic"]["security_type"],
            "sn": results["ssd_config_dic"]["drive_sn"],
            "cap": self.convert_t(results["ssd_config_dic"]["drive_tnvmcap"]),
            "bb": "{}".format(results["drive_life_info_dic"]["count_grown_defects"]),
            "max_ec": results["drive_life_info_dic"]["nand_max_erase_count"],
            "avg_ec": results["drive_life_info_dic"]["nand_avg_erase_count"],
            "agent": self.agent_id,
            "status": SlotState.Idle
        }
        return format_result

    @staticmethod
    def convert_t(cap):
        return "" if cap == "" else float('%.2f' % (cap/1000/1000/1000/1000))

    @update_slot
    def refresh(self, slot_id, config_name):
        result = None
        self.get_orig_logs()
        ret = self.execute_command(config_name)
        if ret == 0:
            result = self.load_result()
        return slot_id, result



#!/usr/bin/env python

import os
import sys
import re
import time
import socket
import yaml
from subprocess import Popen, PIPE, STDOUT
from utils import log


def execute(cmd, cmdline=True, console=True, interrupt=True):
    """
    Execute shell command
    """
    cmd = " ".join(cmd) if isinstance(cmd, list) else cmd
    if cmdline:
        print("# {cmd}".format(cmd=cmd))

    if "linux" in sys.platform:
        proc = Popen(cmd, stdout=PIPE, stderr=STDOUT, shell=True, close_fds=True)
    else:
        proc = Popen(cmd, stdout=PIPE, stderr=STDOUT, shell=True)
    output = ""
    while True:
        line = proc.stdout.readline().decode("utf-8", "ignore")
        if line:
            if console:
                sys.stdout.write(line)
                sys.stdout.flush()
            output += line

        if proc.poll() is not None and line == "":
            break
    status = proc.returncode

    if status and interrupt:
        raise RuntimeError("{} failed!".format(cmd))

    return status, output


def get_ip_address():
    """Get ip address of host"""
    sock = None
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.connect(('8.8.8.8', 80))
        ip_address = sock.getsockname()[0]
    finally:
        if sock:
            sock.close()
    return ip_address


def get_multi_ip_address():
    address = set()
    add_info = socket.getaddrinfo(socket.gethostname(), None)
    for item in add_info:
        if ':' not in item[4][0]:
            address.add(item[4][0])
    return address


def get_host_name():
    name = socket.gethostname()
    return name


def get_root_path():
    root_path = os.path.join(os.path.dirname(__file__), "..")
    return root_path


def get_expect_version(tool_name):
    conf = os.path.join(get_root_path(), "configuration", "version.yaml")
    with open(conf) as file_:
        cont = file_.read()
    cf_ = yaml.load(cont)
    version_ = cf_[tool_name]
    return version_


def version_check(tool):
    act_version = tool.get_version()
    exp_version = get_expect_version(tool.name)
    log.INFO("%s actual version:%s , expect version:%s", tool.name, act_version, exp_version)
    ret = True if act_version == exp_version else False
    return ret


def get_time_stamp():
    return time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime(time.time()))


def nexus_or_nvme_device():
    cmd = "lsblk"
    _, outs = execute(cmd)
    if "nvme" in outs:
        ret = "nvme"
    elif "nexus" in outs:
        ret = "nexus"
    else:
        ret = None
    return ret


def decorate_exception(func):
    def func_wrapper(*args, **kwargs):
        try:
            ret = func(*args, **kwargs)
        except Exception as e:
            print(e)
            ret = False
        return ret
    return func_wrapper


def get_platform():
    platform = os.environ.get('platform', '')
    if platform == "oakgate":
        slot_type = "oakgate"
    elif platform == "linux":
        slot_type = "linux"
    else:
        if "win" in sys.platform.lower():
            slot_type = "oakgate"
        else:
            slot_type = "linux"
    return slot_type


def get_vendor_name(vendor_id):
    print("vendor: input {}".format(vendor_id))
    vendor_id = hex(int(vendor_id)).replace("0x", "")
    vendor_name = ""
    pci_ids_file = os.path.join(os.path.dirname(__file__), "..",  "configuration", "pci.ids")
    with open(pci_ids_file, encoding='UTF-8') as pci_file:
        while True:
            line = pci_file.readline()
            if line:
                if not line.startswith("\t"):
                    if vendor_id.lower() in line.lower():
                        vendor_name = line.split(vendor_id)[1].strip()
                        break
            else:
                break
    return vendor_name


ROOT_PATH = get_root_path()




import threading
import time
import os
import string
import random
from datetime import datetime, timezone, timedelta
from test_framework.test_case import TestCase
from test_framework.test_suite import TestSuite
from test_framework.firmware_download import FirmwareDownloader
from rest_server.resource.models.helper import MyThread
from test_framework.state import State
from test_framework.state import TestType
from test_framework.database.database import decorate_add_tests, decorate_update_test_state
from test_framework.database.dut_database import agent_heart_beat
from test_framework.state import AgentState
from test_framework.dut.dut import Dut
from utils import log


def singleton(cls):
    _instance = {}

    def inner():
        if cls not in _instance:
            _instance[cls] = cls()
        return _instance[cls]
    return inner


@singleton
class TestPool(object):

    def __init__(self):
        self.thread_pool = list()
        self.test_pool = list()
        self._stop_flag = True
        self.run_manage_thread()
        self.all_test_case = self.get_all_test_case()
        self.all_test_suite = self.get_all_test_suite()

    @staticmethod
    def refresh_dut():
        dut = Dut()
        dut.refresh()

    @property
    def stop_flag(self):
        return self._stop_flag

    @stop_flag.setter
    def stop_flag(self, stop_flag):
        self._stop_flag = stop_flag
        log.INFO(type(self._stop_flag))
        log.INFO("set stop flag to {}".format(self._stop_flag))

    def get_all_test_case(self):
        test_case = TestCase()
        lists = test_case.list_and_filter_tests("all")
        return lists

    def get_all_test_suite(self):
        test_suite = TestSuite()
        lists = test_suite.list_and_filter_tests("all")
        return lists

    def run_manage_thread(self):
        thread_p = threading.Thread(target=self.thread_pool_manage)
        thread_p.setDaemon(True)
        thread_p.start()
        thread_t = threading.Thread(target=self.test_pool_manage)
        thread_t.setDaemon(True)
        thread_t.start()
        thread_t = threading.Thread(target=self.dut_state_manage)
        thread_t.setDaemon(True)
        thread_t.start()

    def thread_pool_manage(self):
        while True:
            for index, thread_ in enumerate(self.thread_pool):
                if thread_["state"] == State.NOT_START:
                    p_thread, test_object = self._start_test_thread(thread_)
                    self.thread_pool[index]["pthread"] = p_thread
                    self.thread_pool[index]["test_object"] = test_object
                    self.thread_pool[index]["state"] = State.RUNNING
                    self._update_test_pool_state(thread_["key"], State.RUNNING, None)
                elif thread_["state"] == State.RUNNING:
                    if thread_["pthread"].is_alive() is False:
                        test_result = thread_["pthread"].get_result()
                        state = self.get_state_from_result(test_result)
                        self.thread_pool[index]["state"] = state
                        if (state == State.FAIL) and (self._stop_flag is True):
                            log.INFO("Test failed, abort not start case")
                            self._abort_not_start_tests()
                        self._update_test_pool_state(thread_["key"], state, test_result)
                elif thread_["state"] in [State.PASS, State.FAIL, State.BLOCK, State.ERROR_UNHEALTHY]:
                    self.thread_pool.pop(index)
                elif thread_["state"] == State.ABORT:
                    self.thread_pool.pop(index)
            time.sleep(2)

    def get_state_from_result(self, results):
        if results is not None:
            if len(results) > 1:
                fail_result = [result for result in results if result["result"] is State.FAIL]
                state = State.FAIL if fail_result else State.PASS
            else:
                state = results[0]["result"]
        else:
            state = State.ERROR_BASE_EXCEPTION
        return state

    def test_pool_manage(self):
        while True:
            time.sleep(5)
            try:
                self.add_new_test_to_run()
            except Exception as all_exception:
                log.ERR(all_exception)

    def dut_state_manage(self):
        self.refresh_dut()
        while True:
            time.sleep(5)
            try:
                self.timing_refresh_dut()
                self.heart_beat_check_env_state()
            except Exception as all_exception:
                log.ERR(all_exception)

    def add_new_test_to_run(self):
        ret = self._has_running_test()
        if ret is False:
            for index, test in enumerate(self.test_pool):
                if test["state"] == State.NONE:
                    self._add_test_to_thread_pool(test)
                    self.test_pool[index]["state"] = State.NOT_START
                    break

    def timing_refresh_dut(self):
        now = datetime.now(timezone(timedelta(hours=+8)))
        if (now.hour == 11 and now.minute == 59 and now.second in range(49, 59)) or\
                (now.hour == 12 and now.minute == 0 and now.second in range(0, 10)):
            log.INFO("Time refresh dut")
            self.refresh_dut()

    def get_current_agent_state(self):
        tests = [test for test in self.test_pool if test["state"] in [State.NOT_START, State.RUNNING, State.NONE]]
        if tests:
            agent_state = AgentState.Running
        else:
            agent_state = AgentState.Online
        return agent_state

    def _add_test_to_thread_pool(self, test):
        test_thread = {
            "key": test["key"],
            "pthread": None,
            "test_object": None,
            "test_name": test["test_name"],
            "test_type": test["test_type"],
            "state": State.NOT_START,
            "thread_type": "test",
            "test_parameters": test["test_parameters"]
            }
        log.INFO("add test case {}".format(test["test_name"]))
        self.thread_pool.append(test_thread)

    def _has_running_test(self):
        result = True if self.thread_pool else False
        return result

    def _abort_not_start_tests(self, key=None):
        for test_ in self.test_pool:
            if test_["state"] in [State.NOT_START, State.NONE]:
                if key is None or test_["key"] == key:
                    self._update_test_pool_state(test_["key"], State.ABORT, None)

    @decorate_update_test_state
    def _update_test_pool_state(self, test_key, state, test_result):
        for index, thread_ in enumerate(self.test_pool):
            if thread_["key"] == test_key:
                self.test_pool[index]["state"] = state
                self.test_pool[index]["test_result"] = test_result

    def _start_test_thread(self, thread_info):
        test_name = thread_info["test_name"]
        test_type = thread_info["test_type"]
        test_parameters = thread_info["test_parameters"]
        thread_ = None
        test_object = None
        if test_type == TestType.TestCase:
            self.set_test_parameters(test_parameters)
            thread_, test_object = self._start_test_case_thread(test_name, test_parameters)
        elif test_type == TestType.TestSuite:
            self.set_test_parameters(test_parameters)
            thread_, test_object = self._start_test_suite_thread(test_name, test_parameters)
        elif test_type == TestType.DownloadSerial:
            thread_, test_object = self._start_download_serial_thread(test_name, test_parameters)
        return thread_, test_object

    def _start_test_suite_thread(self, test_name, test_parameters):
        test_suite = TestSuite()
        thread_ts = MyThread(target=test_suite.run, args=(test_name, test_parameters,))
        thread_ts.setDaemon(True)
        thread_ts.start()
        return thread_ts, test_suite

    def _start_test_case_thread(self, test_name, test_parameters):
        test_case = TestCase(test_name)
        thread_tc = MyThread(target=test_case.run, args=(test_name, test_parameters,))
        thread_tc.setDaemon(True)
        thread_tc.start()
        return thread_tc, test_case

    def _start_download_serial_thread(self, execute_name, test_parameters):
        downloader = FirmwareDownloader(execute_name)
        thread_tc = MyThread(target=downloader.run, args=(test_parameters,))
        thread_tc.setDaemon(True)
        thread_tc.start()
        return thread_tc, downloader

    def _get_unique_code(self):
        letter_len = 10
        code = ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(letter_len))
        return code

    def _stop_running_test_process(self, test_object):
        ret = 0
        if test_object is not None:
            ret = test_object.stop()
        return ret

    def _stop_running_tests(self):
        result = True
        ret_stop_test = -1
        for index, thread_ in enumerate(self.thread_pool):
            print(thread_)
            if thread_["thread_type"]=="test" and thread_["state"]==State.RUNNING:
                test_object = thread_["test_object"]
                self.thread_pool[index]["state"] = State.ABORT
                self._update_test_pool_state(thread_["key"], State.ABORT, None)
                for index_ in range(10):
                    ret_stop_test = self._stop_running_test_process(test_object)
                    log.INFO("Stop test thread loop %s, ret %s", index_, ret_stop_test)
                    if ret_stop_test == 0:
                        break
                time.sleep(1)
                count = 0
                while thread_["pthread"].is_alive() is True and count < 2:
                    log.INFO("Stop thread loop ")
                    thread_["pthread"].stop()
                    time.sleep(1)
                    count = count + 1
        return result

    def _check_test_is_exist(self, test_name, test_type):
        if test_type == TestType.TestSuite:
            ret = True if test_name in self.all_test_suite else False
        elif test_type == TestType.TestCase:
            ret = True if test_name in self.all_test_case else False
        else:
            ret = True
        return ret

    @decorate_add_tests
    def add_test(self, test_name, test_type, test_parameters=None):
        if self._check_test_is_exist(test_name, test_type):
            unique_code = self._get_unique_code()
            if test_parameters is not None:
                test_parameters["key"] = unique_code
            test_dict = {"test_name": test_name,
                         "test_type": test_type,
                         "state": State.NONE,
                         "test_result": None,
                         "thread_type": "test",
                         "key": unique_code,
                         "test_parameters": test_parameters}
            self.test_pool.append(test_dict)
        else:
            unique_code = None
        return unique_code

    def stop_test(self, key=None):
        if key is None:
            result = self._stop_all_tests()
        else:
            result = self._stop_test_by_key(key)
        return result

    def _stop_all_tests(self):
        self._abort_not_start_tests()
        result = self._stop_running_tests()
        return result

    def _stop_test_by_key(self, key):
        result = True
        test_case = self.find_test_state_by_key(key)
        if test_case is not None:
            test_state = test_case["state"]
            if test_state in [State.NOT_START, State.NONE]:
                self._abort_not_start_tests(key)
            elif test_state == State.RUNNING:
                self._abort_not_start_tests(key)
                result = self._stop_running_tests()
        return result

    def find_test_state_by_key(self, key):
        test_case = None
        for index, test_ in enumerate(self.test_pool):
            if self.test_pool[index]["key"] == key:
                test_case = test_
        return test_case

    def get_state_by_key(self, key):
        test = list(filter(lambda X: X["key"] == key, self.test_pool))
        if test:
            result = test[0]["test_result"]
            state = test[0]["state"]
        else:
            log.ERR("Test:%s, did not found", key)
            state = State.ERROR_NOT_FOUND
            result = None
        return result, state

    @agent_heart_beat
    def heart_beat_check_env_state(self):
        status = self.get_current_agent_state()
        return status

    @staticmethod
    def set_test_parameters(parameters):
        if type(parameters) is dict:
            for key in parameters.keys():
                os.environ[key] = parameters[key]

    def test_set_parameters(self, parameters):
        print("Custom global parameters: ")
        if type(parameters) is dict:
            for key in parameters.keys():
                print("parameters: ", key, os.environ[key])


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# pylint: disable=superfluous-parens,missing-docstring,too-few-public-methods
"""
Created on Apr 19, 2019

@author: yyang
"""
from ctypes import Structure, c_uint16, c_uint32, c_uint64

class BLKSTAT():
    FREE = 0x00
    OPEN = 0x01
    CLOSE = 0x02
    RECYCLE = 0x06
    ERASE = 0x03
    PROTECT = 0x07

class TahoeBMIEntry(Structure):
    """bmi format"""
    _pack_ = 1
    _fields_ = [
        ('bm_st', c_uint32, 3),
        ('blk_type', c_uint32, 3),
        ('bad', c_uint32, 1),
        ('rsv0', c_uint32, 1),
        ('ecc_lvl', c_uint32, 4),
        ('repl_flag', c_uint32, 1),
        ('all_good', c_uint32, 1),
        ('xlc', c_uint32, 2),
        ('gc_class', c_uint32, 8),
        ('wf_flg', c_uint32, 1),
        ('rf_flg', c_uint32, 1),
        ('ecc_hi', c_uint32, 1),
        ('ecc_un', c_uint32, 1),
        ('dr_flg', c_uint32, 1),
        ('wl_flg', c_uint32, 1),
        ('rdist', c_uint32, 1),
        ('gc_soft', c_uint32, 1),
        ('nex_ptr', c_uint32, 16),
        ('mbu_cnt', c_uint32, 10),
        ('rsv1', c_uint32, 6),
        ('cr_tm', c_uint32),
        ('vpc', c_uint32),
        ('rd_cnt', c_uint32),
        ('seq_num', c_uint32),
        ('pe_cyc', c_uint32, 20),
        ('rsv2', c_uint32, 12),
        ('strem_id', c_uint32, 9),
        ('rsv3', c_uint32, 7),
        ('ext_lrb', c_uint32, 16),
        ('wfb0', c_uint32, 16),
        ('wfb1', c_uint32, 16),
        ('rfb0', c_uint32, 16),
        ('rfb1', c_uint32, 16),
        ('thr_cfg', c_uint32),
        ('rsv4', c_uint32),
        ('badb', c_uint16 * 32),
        ('rsv5', c_uint32),
        ('rsv6', c_uint64),
        ('rsv7', c_uint32, 16),
        ('hamming_ecc', c_uint32, 16),
    ]

class TahoeVUPower(Structure):
    """bmi format"""
    _pack_ = 1
    _fields_ = [
        ('vol', c_uint32, 16),
        ('cur', c_uint32, 16),
        ('max_credit', c_uint32),
        ('min_credit', c_uint32),
        ('avg_credit', c_uint32),
        ('max_rd_die', c_uint32),
        ('min_rd_die', c_uint32),
        ('avg_rd_die', c_uint32),
        ('max_wr_die', c_uint32),
        ('min_wr_die', c_uint32),
        ('avg_wr_die', c_uint32),
        ('max_ers_die', c_uint32),
        ('min_ers_die', c_uint32),
        ('avg_ers_die', c_uint32),
    ]




from test_framework.database.dut_database import update_slot, check_lost_slot
from tool.device.linux_nvme import LinuxNvme
from test_framework.state import SlotState
from utils.system import get_vendor_name


class LinuxSlot(object):

    def __init__(self, db_slots, agent_id):
        self.db_slots = db_slots
        self.live_slots = list()
        self.agent_id = agent_id
        self.nvme = LinuxNvme()

    @update_slot
    def refresh(self, device):
        dev_info = self.nvme.get_info(int(device["ctrl_id"]))
        slot_info = self._format_results(dev_info, device)
        slot_id = self.get_slot_id(slot_info)
        print("refresh linux device", slot_id, slot_info)
        return slot_id, slot_info

    def get_slot_id(self, device):
        slot_id = -1
        for item in self.db_slots:
            if item[3] == device["slot"]:
                slot_id = item[0]
                self.live_slots.append(slot_id)
        return slot_id

    def refresh_all(self):
        devices = self.nvme.get_linux_nvme_devs()
        for item in devices:
            self.refresh(item)
        self.check_lost_slots()

    @check_lost_slot
    def is_slot_lost(self, slot):
        if slot[0] not in self.live_slots:
            return slot[0]
        else:
            return None

    def check_lost_slots(self):
        for item in self.db_slots:
            self.is_slot_lost(item)

    def _format_results(self, results, dev):
        format_result = {
            "slot": dev["name"],
            "vendor": get_vendor_name(results["ssd_config_dic"]["vid"]),
            "fw_version": results["test_fw_config_dic"]["fw_public_revision"],
            "commit": results["test_fw_config_dic"]["fw_private_revision"][0:6],
            "ise/sed": results["ssd_config_dic"]["security_type"],
            "sn": results["ssd_config_dic"]["drive_sn"],
            "cap": self.convert_t(results["ssd_config_dic"]["drive_tnvmcap"]),
            "bb": results["drive_life_info_dic"]["count_grown_defects"],
            "max_ec": results["drive_life_info_dic"]["nand_max_erase_count"],
            "avg_ec": results["drive_life_info_dic"]["nand_avg_erase_count"],
            "agent": self.agent_id,
            "status": SlotState.Idle
        }
        return format_result

    @staticmethod
    def convert_t(cap):
        return float('%.2f' % (cap/1000/1000/1000/1000))



# coding=utf-8
# pylint: disable=inconsistent-return-statements
import os
import re
import sys
import importlib
import inspect
from utils import log


class TestBase(object):

    def __init__(self):
        self.working_path = os.getcwd()
        self.test_case_path = self.get_test_case_path()
        self.modules = []
        self.tests = []
        self.module_list = []

    def get_test_case_path(self):
        test_case_path = None
        dirs = os.listdir(self.working_path)
        for item in dirs:
            temp_path = os.path.join(self.working_path, item)
            if "TestScripts" in item or "testcase" == item.lower():
                test_case_path = temp_path
        return test_case_path

    def get_all_script_path(self, script):
        list_script = script.split(';')
        path_list = []
        for single_script in list_script:
            path = self._get_full_path_script(single_script.strip())
            if path is not None:
                path_list.append(path)
        return path_list

    def _get_full_path_script(self, script):
        if script.find(':') != -1:
            file_name = script.split(':')
            if file_name[0].find('.py') == -1:
                file_path = self.get_file_path(self.test_case_path, file_name[0]+'.py')
            else:
                file_path = self.get_file_path(self.test_case_path, file_name[0])
            if file_path is not None:
                abs_path = file_path + ':' + file_name[1]
            else:
                abs_path = None
        else:
            abs_path = self.get_file_path(self.test_case_path, script)
        return abs_path

    def get_file_path(self, path, file_):
        _files = os.listdir(path)
        for item in _files:
            item_path = os.path.join(path, item)
            if os.path.isdir(item_path):
                ret = self.get_file_path(item_path, file_)
                if ret is not None:
                    return ret
            elif item == file_:
                return os.path.join(path, item)

    def get_all_modules(self, path):
        if path is None or os.path.exists(path) is False:
            return
        _files = os.listdir(path)
        for item in _files:
            item_path = os.path.join(path, item)
            if os.path.isdir(item_path):
                ret = self.get_all_modules(item_path)
                if ret is not None:
                    return ret
            elif ".py" in item and ("test" in item.lower() or "tc_" in item  or "mc_" in item)and "__init__" not in item and ".pyc" not in item:
                self.module_list.append(item_path)

    def list_tests(self):
        for item in self.module_list:
            self.list_test_in_one_file(item)

    def list_test_in_one_file(self, file_path):
        module_name = os.path.basename(file_path).split(".")[0]
        if str(module_name).startswith("tc_") is True or str(module_name).startswith("mc_") is True:
            self.tests.append(os.path.basename(file_path))   #oakgate test case
        else:
            file_ = open(file_path, encoding='UTF-8')
            class_name = None
            for line_ in file_.readlines():
                # get_class_name = self.get_class_name(line_)
                get_class_name, base_class_name = self.get_class_subclass_name(line_)
                self.add_base_class_tests(module_name, base_class_name, get_class_name)
                if get_class_name is not None:
                    class_name = get_class_name
                    test = "{}:{}".format(module_name, class_name)
                    self.tests.append(test)
                else:
                    get_function_name = self.get_function_name(line_)
                    if get_function_name:
                        test = "{}:{}.{}".format(module_name, class_name, get_function_name[0])
                        self.tests.append(test)
            file_.close()

    def add_base_class_tests(self, module_name, base_class_name, class_name):
        new_tests = list()
        if base_class_name not in ["", "object", None]:
            base_test = "{}:{}".format(module_name, base_class_name)
            for test in self.tests:
                test_fun = test.split(".")
                if len(test_fun) == 2:
                    if test_fun[0] == base_test:
                        if test_fun:
                            try:
                                new_test = "{}:{}.{}".format(module_name, class_name, test_fun[1])
                                new_tests.append(new_test)
                            except Exception as all_exception:
                                print(all_exception)
        self.tests.extend(new_tests)

    def get_class_name(self, line):
        rets = re.findall("^\s*class\s+(Test[a-zA-Z0-9\_]+)\(", line)
        return rets

    def get_class_subclass_name(self, line):
        class_name, base_class_name = None, None
        rets = re.findall("^\s*class\s+([a-zA-Z0-9\_]+)\((.*?)\)", line)
        if rets:
            class_name, base_class_name = rets[0][0], rets[0][1]
        return class_name, base_class_name

    def get_function_name(self, line):
        rets = re.findall("^\s+def\s+(test[a-zA-Z0-9\_]+)\(", line)
        return rets

    def test(self):
        return self.tests

    def get_log_path(self):
        log_path = os.path.join(self.working_path, "log")
        if os.path.exists(log_path) is False:
            os.mkdir(log_path)
        return log_path



import subprocess
import os
import re
from utils.message import Message
from utils.system import get_ip_address


class GitOperator(object):

    def __init__(self, user, key):
        self.project_path = os.environ["working_path"]
        self.user = user
        self.key = key
        self.message = Message("Update git operation logs on {}".format(get_ip_address()))

    def _execute_git_command(self, cmd):
        new_command_line = "cd /d {} && {}".format(self.project_path, cmd)
        print(new_command_line)
        process = subprocess.Popen(new_command_line, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        (std_output, std_error) = process.communicate()
        ret = process.poll()
        self.message.add_message("Execute: {}\n{}\n{}".format(cmd, std_output.decode("utf-8"), std_error.decode("utf-8")))
        return std_output, ret

    def hard_reset(self):
        cmd = "git reset --hard"
        std_output, ret = self._execute_git_command(cmd)
        return std_output, ret

    def get_project_remote_url(self):
        cmd = "git remote -v"
        std_output, ret = self._execute_git_command(cmd)
        links = re.findall("(http.*?\.git)", std_output.decode("utf-8"))
        link = links[0] if links else ""
        return link

    def adjust_remote_url(self, orginal_url):
        new_url = orginal_url
        if "@" in orginal_url:
            temps = orginal_url.split("@")
            new_url = "https://"+temps[1]
            self.git_config_remote_url(new_url)
        return new_url

    def git_config_remote_url(self, remote_url):
        command = "git config remote.origin.url {}".format(remote_url)
        self._execute_git_command(command)

    def update_code(self, target_version):
        ret = -1
        remote_url = self.get_project_remote_url()
        remote_url = self.adjust_remote_url(remote_url)
        if remote_url != "":
            authentication = "//{}:{}@".format(self.user, self.key)
            remote_url = remote_url.replace("//", authentication)
            self.rev_parse()
            ret = self.fetch_code(remote_url)
            if ret == 0:
                commit_id = self.get_target_commit_id(target_version)
                if commit_id != "":
                    ret = self.git_check_out(commit_id)
        return ret

    def rev_parse(self):
        command_line = "git rev-parse --is-inside-work-tree"
        self._execute_git_command(command_line)

    def fetch_code(self, remote_url):
        command_line = "git fetch --tags --progress {} +refs/heads/*:refs/remotes/origin/*".format(remote_url)
        _, ret = self._execute_git_command(command_line)
        return ret

    def git_check_out(self, commit_id):
        command_line = "git checkout -f {}".format(commit_id)
        _, ret = self._execute_git_command(command_line)
        return ret

    def get_target_commit_id(self, target_version):
        command_line = "git rev-parse {}".format(target_version)
        std_output, ret = self._execute_git_command(command_line)
        commit_id = std_output[0:8] if ret==0 else b""
        return commit_id.decode("utf-8")

    def update_latest_code(self, target_version):
        _, ret = self.hard_reset()
        if ret == 0 :
            self.message.add_message("Git hard reset succeed")
            ret = self.update_code(target_version)
        else:
            self.message.add_message("Git hard reset Failed, skip update code ")
        new_msg = re.sub("\n+", "\n", self.message.message)
        return new_msg, ret
